''' 
This module contains helper functions to load kepler survey data into EPOS
'''
import numpy as np
import os
import EPOS

try:
	from astropy.table import Table
except ImportError:
	print '\nWarning: Could not import astropy.table'

fpath= os.path.dirname(EPOS.__file__)

def dr25(subsample='all', score=0.9, Gaia=False, Huber=True, Vetting=False):
	'''
	Generates Kepler DR25 planet population and detection efficiency
	
	Args:
		subsample(str):	Subsample, choose from 'all', 'M', 'K', 'G', or 'F'
		score(float): Disposition score, 0-1, default 0.9
		Gaia(bool): Use Gaia data (Stellar radii from Berger+ 2018)
		Huber(bool): Use logg cut from Huber+ 2016
		Vetting(bool): include vetting completeness
		
	Returns:
		tuple: two dictionaries
		
		obs(dict):
			xvar(np.array of float): orbital period
			yvar(np.array of float): planet radius
			starID(np.array): stellar ID
			nstars(int): number of stars surveyed
		survey(dict):
			the grid is in xvar,yvar, the detection efficiency in eff_2D 
	'''
	if not os.path.isdir('temp/'): os.makedirs('temp/')

	if Gaia:
		#fkoi= 'temp/q1_q17_dr25-gaia-r1_koi.npz'
		fkoi= 'temp/q1_q17_dr25-gaia-r2_koi.npz'
	else:
		fkoi= 'temp/q1_q17_dr25_koi.npz'
		
	if os.path.isfile(fkoi):
		print '\nLoading planets from {}'.format(fkoi)
		koi= np.load(fkoi)
	else:
		print '\nReading planet candidates from IPAC file' 
		try:
			ipac=Table.read(fpath+'/files/q1_q17_dr25_koi.tbl',format='ipac')
		except NameError:
			raise ImportError('You need to install astropy for this step')
		#print ipac.keys()
		nonzero= (ipac['koi_srad']>0)
		nremove= ipac['koi_srad'].size- ipac['koi_srad'][nonzero].size
		print '  removed {} planets with no stellar radii'.format(nremove)
		koi= {key: np.array(ipac[key][nonzero]) for key in 
				['kepid','koi_prad','koi_period',
				'koi_steff', 'koi_slogg', 'koi_srad', 'koi_depth',
				'koi_pdisposition','koi_score'] }
		# isnan= ~(koi['koi_srad']>0)
		# print isnan.size
		# print koi['kepid'][isnan]
		# print koi['koi_srad'][isnan]
		# print koi['koi_pdisposition'][isnan]
		# print koi['koi_prad'][isnan]

		if Gaia:
			# Stellar radius table from Berger+ in prep., revision 2
			#fgaia= 'files/DR2PapTable1_v1.txt'
			fgaia= '{}/files/DR2PapTable1.txt'.format(fpath)
			a=np.loadtxt(fgaia, delimiter='&', unpack=True, skiprows=1, comments='\\')
			
			# sort stars by ID for easy matching
			order= a[0].argsort()
			a= a[:,order]
			assert np.all(np.sort(a[0])==a[0]), 'Whoops II'
			
			common= np.intersect1d(koi['kepid'], a[0], assume_unique=False)
			print '  {} common out {} kois and {} stars in gaia'.format(common.size, 
							koi['kepid'].size, a[0].size)
			
			# remove kois w/ no gaia data
			koi_in_gaia= np.in1d(koi['kepid'], a[0])
			for key in koi:
				koi[key]= koi[key][koi_in_gaia]
			print '  {} stars, {} kois in gaia'.format(koi['kepid'].size, 
				np.unique(koi['kepid']).size)
			
			# remove gaia data w/ no koi
			gaia_in_koi= np.in1d(a[0], koi['kepid'])
			gaia=dict(starID= a[0][gaia_in_koi],
					  Teff=a[2][gaia_in_koi],
					  distance=a[4][gaia_in_koi],
					  Rst= a[7][gaia_in_koi],
					  giantflag= a[11][gaia_in_koi])
			print '  {} gaia stars that are kois'.format(gaia['starID'].size)
			assert gaia['starID'].size == np.unique(koi['kepid']).size
			
			# stars w/ multiple planet candidates		
			_, st_to_pl= np.unique(koi['kepid'], return_inverse=True)
			assert np.all(gaia['starID'][st_to_pl]==koi['kepid'])
			
			# update radii
			with np.errstate(divide='ignore'):
				increase= np.nanmedian(gaia['Rst'][st_to_pl]/ koi['koi_srad'])-1.
			print '  KOI radii increased by {:.1%}'.format(increase)
			
			koi['koi_steff']= gaia['Teff'][st_to_pl]
			with np.errstate(divide='ignore', invalid='ignore'):
				koi['koi_prad']*= gaia['Rst'][st_to_pl]/ koi['koi_srad']
			koi['distance']= gaia['distance'][st_to_pl]
			koi['giantflag']= gaia['giantflag'][st_to_pl]

			#print np.nanmedian((koi['koi_srad']*koi['koi_depth']**0.5)/ koi['koi_prad'])
		
		np.savez(fkoi, **koi)

	''' Remove giant stars'''
	if Gaia:
		isdwarf= koi['giantflag'] == 0
		suffix='gaia-r1' # completeness needs update to r2
	elif Huber:
		isdwarf= koi['koi_slogg'] > 1./4.671 * \
					np.arctan((koi['koi_steff']-6300.)/-67.172)+ 3.876
		isgiant= koi['koi_slogg'] < np.where(koi['koi_steff']>5000, 
									13.463-0.00191*koi['koi_steff'],3.9)
		issubgiant= ~isdwarf & ~isgiant
		suffix='huber'
	else:
		isdwarf= koi['koi_slogg']>4.2
		suffix='logg42'
	
	''' Select reliable candidates '''
	iscandidate= koi['koi_pdisposition']=='CANDIDATE'
	isreliable= koi['koi_score']>=score # removes rolling band, ~ 500 planets
	print '  {}/{} dwarfs'.format(isdwarf.sum(), isdwarf.size)
	print '  {} candidates, {} false positives'.format((isdwarf&iscandidate).sum(), 
				(isdwarf&~iscandidate).sum() )
	print '  {}+{} with score > {:.2f}'.format((isdwarf&iscandidate&isreliable).sum(), 
				(isdwarf&~iscandidate&isreliable).sum(),score )

	if score >= 0.5:
		isall= isdwarf & isreliable # reliability score cuts out false positives
	else:
		isall= isdwarf & iscandidate & isreliable

	''' Select a spectral type sub-smaple'''
	if subsample=='all':
		slice=isall
	elif subsample in ['M','K','G','F']:
		Teff={'M':[2400, 3865], 'K':[3865, 5310], 'G':[5310, 5980], 'F':[5980, 7320]}
		slice= isall & (Teff[subsample][0]<koi['koi_steff']) \
						& (koi['koi_steff']<=Teff[subsample][1])
	elif subsample[0] == 'T':
		Tmin=int(subsample[1:])-250
		Tmax=int(subsample[1:])+250
		slice= isall & (Tmin<koi['koi_steff']) & (koi['koi_steff']<=Tmax)
	else:
		raise ValueError('Subsample {} not recognized'.format(subsample))
	#if Huber: slice['subgiants']=issubgiant
	
	obs= {'xvar':koi['koi_period'][slice],
		'yvar':koi['koi_prad'][slice], 
		'starID':koi['kepid'][slice],
		'score':koi['koi_score'][slice]}
	
	''' Load pre-calculated detection efficiencies '''
	# from dr25_epos.py
	sfile= 'dwarfs' if subsample=='all' else subsample
	eff= np.load('{}/files/completeness.dr25.{}.{}.npz'.format(fpath,sfile, suffix))
	#eff= np.load('files/det_eff.dr25.{}.npz'.format(subsample))
	survey= {'xvar':eff['P'], 'yvar':eff['Rp'], 'eff_2D':eff['fsnr'], 
			'Mstar': eff['Mst'], 'Rstar':eff['Rst']}
	obs['nstars']= eff['n']
  	
	''' Add vetting completeness '''
	if Vetting:
		X,Y= np.meshgrid(eff['P'], eff['Rp'], indexing='ij')
		if subsample=='all' and score == 0.9:
			if Gaia:
				vetpars= 0.84, 55., -0.07, -0.37, 8.2, 0.13, 0.13
			else:
				vetpars= 0.88, 53., -0.07, -0.39, 5.7, 0.19, 0.19
								
		elif subsample=='all' and score == 0.0:
			if Gaia:
				vetpars= 0.88, 210, -0.0035, -0.24, 6.9, -0.029, -0.029
			else:
				vetpars= 0.86, 210, 0.002, -0.22, 5.5, -0.057, -0.057
		else:
			raise ValueError('no vetting completeness for {} with score={}'.format(subsample, score))
		
		vet_2D= fbpl2d( (X,Y), *vetpars)
		survey['vet_2D']= vet_2D
		assert vet_2D.shape == eff['fsnr'].shape

	return obs, survey
	
def fbpl2d((x,y), a, b, c, d, e, f, g):
	bpl= a* (x/b)**np.where(x<b, c, d) * (y/e)**np.where(y<e, f,g)
	return np.maximum(0.2, np.minimum(bpl, 1.))

def single():
	'''
	Generates Kepler single transit planet population and detection efficiency
	
	Args:
		
	Returns:
		tuple: two dictionaries
		
		obs(dict):
			xvar(np.array of float): orbital period
			yvar(np.array of float): planet radius
			starID(np.array): stellar ID
			nstars(int): number of stars surveyed
		survey(dict):
			the grid is in xvar,yvar, the detection efficiency in eff_2D 
	'''
	P_days= np.array([1246.35,1071.23,1772.14,1018.14,1047.83,2608.45,704.20,
		730.83,1006.63,769.19,8375.64,1183.93,936.06,732.02,737.11,741.58, 
		1245.41,854.09,1632.13])
	R_Jup= np.array([0.783,0.904,0.662,0.374,0.727,0.310,0.411,3.296,0.577,2.368,
		0.580, 0.926, 0.733,2.298,0.955,2.713,0.464,0.541,0.889])
	starID=np.array(['3218908','3239945','4754460','6551440','8410697','8505215',
		'8800954','9306307','10187159','10602068','10842718','11709124',
		'6186417','6234593','7906827','7947784','9704149','10525077','11342550'])

	obs= {'xvar':P_days,'yvar':R_Jup, 'starID':starID,'nstars':61418}
	
	''' Load pre-calculated detection efficiencies '''
	xvar= np.geomspace(2,25,5) # 4 bins :@@
	yvar= np.logspace(0.02,0.3,8) # 7 bins

	eff_2d= np.array(
		[[0.542,0.485,0.461,0.374],[0.638,0.569,0.537,0.472],
		[0.704,0.601,0.607,0.532],[0.645,0.578,0.543,0.535],
		[0.497,0.436,0.433,0.384],[0.198,0.142,0.152,0.139],
		[0.049,0.037,0.027,0.016]]
		)

	survey= {'xvar':xvar, 'yvar':yvar, 'eff_2D':eff_2d, 
			'Mstar': 1.0, 'Rstar':1.0}

	return obs, survey


'''
NOTE: Kepler files from Q16-epos.py
cp ~/Kepler/npz/completeness.Q16.epos.*.npz EPOS/files/
cp ~/Kepler/npz/KOI.Q16.epos.*.npz EPOS/files/

#cp ~/Kepler/npdict/Q16.occ.Rp-P.sptype.*.npdict EPOS/files/
'''

def obs_Q16(subsample='all'):
	'''
	returns the Q16 exoplanet catalogue as numpy arrays
	
	Returns:
		P(np.array):   Orbital period in days
		Rp(np.array):  Planet radius in earth radii'
		KID(np.array): star identifier (for multis)'
	'''
	KOI= np.load('files/KOI.Q16.epos.{}.npz'.format(subsample))
	return KOI['P'], KOI['Rp'], KOI['KID']

def eff_Q16(subsample='all'):
	'''
	returns the Q16 survey detection efficiency as a 2D matrix
	
	Returns:
		P(np.array):   Orbital period in days
		Rp(np.array):  Planet radius in earth radii'
		eff(np.array):2D matrix of detection efficiency [0-1]
	'''
	eff= np.load('{}/files/completeness.Q16.epos.{}.npz'.format(fpath,subsample))
	return eff['x'], eff['y'], eff['fsnr']



#! /usr/bin/env ipython
''' 
Estimate plane occurrence rates using the inverse detection efficiency method

Plots should appear in the directory
png/example_9/occurrence/

Binned occurrence rates are saved in 
json/example_9/

This example calculates the occurrence rates for different classes of planets, 
defined by a size and period-range
'''

import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='example_9')

''' load the kepler dr25 exoplanets and survey efficiency '''
obs, survey= EPOS.kepler.dr25(Huber=True, Vetting=True, score=0.9)
epos.set_observation(**obs)
epos.set_survey(**survey)

''' Set plotting ranges (zoom ranges are not used) '''
epos.set_ranges(xtrim=[0.5,830],ytrim=[0.3,20.],xzoom=[20,300],yzoom=[0.7,3])

''' define a set of period (x) and radius (y) bins to calculate occurrence rates'''
x_HJ,y_HJ= [1,10],[6,20] # Hot-Jupiters
x_SEMN, y_SEMN= [2,150],[1.0,4.0] # super-earths and mini-Neptunes

''' Pass the list of bins to EPOS'''
#epos.set_bins(xbins=[x_HJ,x_SEMN], 
#	ybins=[y_HJ,y_SEMN])

import numpy as np
epos.set_bins(xgrid=np.geomspace(10,640,7), 
   	ygrid=np.geomspace(0.67,17,9), Grid=True)

''' Calculate the occurrence rates '''
EPOS.occurrence.all(epos)

''' Save the occurrence rates per bin in '''
EPOS.save.occurrence(epos)

''' Adjust axes to plot habitable zone '''
epos.plotpars['textsize']= 8
epos.xtrim[1]= 1000

''' plot occurrence rates '''
EPOS.plot.occurrence.all(epos)


import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colorbar as clrbar
from matplotlib.colors import Normalize

import helpers
from EPOS import cgs
from EPOS.population import periodradius

def oneD(epos, PlotZoom=False, MCMC=False, Occ=False):
	if not epos.Range: epos.set_ranges()
	
	oneD_x(epos, PlotZoom=PlotZoom, MCMC=MCMC, Occ=Occ)
	oneD_y(epos, PlotZoom=PlotZoom, MCMC=MCMC, Occ=Occ)
	if epos.MassRadius:
		# works with occ??
		oneD_y(epos, PlotZoom=PlotZoom, MCMC=MCMC, PlotQ=True)

def oneD_x(epos, PlotZoom=False, MCMC=False, Occ=False, Log=True):	

	if Occ:
		fname= 'occurrence/posterior' if MCMC else 'occurrence/input'
		ybin= epos.yzoom
		unit= r'$M_\bigoplus$' if epos.RV else r'$R_\bigoplus$'
		title= r'Planet Occurrence ({1:.1f}-{2:.0f} {0})'.format(unit, *epos.yzoom)
	else:	
		fname= 'mcmc/posterior' if MCMC else 'input/parametric_initial'
		ybin=None
		title= r'Marginalized Distribution ({:.1f}-{:.0f} $R_\bigoplus$)'.format(*epos.ytrim)
	
	if not Log: 
		fname+= '.linear'

	# initial guess
	pps, _, pdf0_X, _= periodradius(epos, Init=True, ybin=ybin)
	if MCMC:
		# best-fit parameters
		pps, _, pdf_X, _= periodradius(epos, ybin=ybin)
	
	''' construct the posterior parameters '''
	if MCMC: plotsample= epos.samples[np.random.randint(len(epos.samples), size=100)]
		
	''' Orbital Period '''
	f, ax = plt.subplots()
	ax.set_title(title)
	ax.set_ylabel('Occurrence / {}P'.format(epos.plotpars['area']))

	#ax.set_xlabel('Orbital Period [days]')
	#ax.set_xscale('log')
	#ax.set_xlim(epos.xtrim)
	helpers.set_axis_distance(ax, epos, Trim=True)
	
	if Log:
		ax.set_yscale('log')
		if 'occrange' in epos.plotpars:
			ax.set_ylim(epos.plotpars['occrange'])
		else:	
			ax.set_ylim([1e-3,1e1])
	else:
		ax.set_ylim([0,0.45])
	
	if MCMC:
		for fpara in plotsample:
			_, _, xpdf, _= periodradius(epos, fpara=fpara, ybin=ybin)
			ax.plot(epos.MC_xvar, xpdf, color='b', alpha=0.1)
						
		ax.plot(epos.MC_xvar, pdf0_X, marker='',ls=':',color='k',
					label='Starting Guess')
		ax.plot(epos.MC_xvar, pdf_X, marker='',ls='-',color='k',
					label='Best Fit')
	else:
		if 'P break' in epos.fitpars.keys2d:
			ax.axvline(epos.fitpars.get('P break', Init=True), ls='-', color='gray')
		ax.plot(epos.MC_xvar, pdf0_X, marker='',ls='-',color='k')
    
	# plot posterior excluding low detection regions (arbitrary 3000 planets assumed)
	#if not (epos.RV or epos.MassRadius):
	if False:
		cens= np.where(epos.f_det<1./3000.,0,1.)
		_, _, cens_pdf_X, _ = periodradius(epos, ybin=ybin, fdet=cens)
		ax.plot(epos.MC_xvar, cens_pdf_X, marker='',ls='-',color='green', label='biased')


	if epos.Zoom: 
		for zoom in epos.xzoom: ax.axvline(zoom, ls='--', color='k')
	
	if Occ:
		occbin= epos.occurrence['yzoom']
		ax.errorbar(occbin['xc'], occbin['occ']/occbin['dlnx'], 
			yerr= occbin['err']/occbin['dlnx'], color='r', marker='_', ls='', capsize=3) #,capthick=2

	# lognormal inner disk edge?
	#from scipy.stats import norm
	#gauss= 2.*norm(loc=1., scale=0.4).pdf(np.log10(epos.MC_xvar))
	#ax.plot(epos.MC_xvar, gauss, ls='-', marker='', color='r')

	if not Log:
		if 'Pinner all' in epos.plotpars:
			xx=np.geomspace(20,400)
			a,b,c= epos.plotpars['Pinner all']
			ax.plot(xx, a* (xx/b)**c, marker='', ls='-', color='g', 
				label='All Planets')
		if 'Pinner ylim' in epos.plotpars:
			ax.set_ylim(epos.plotpars['Pinner ylim'])

	if MCMC:
		ax.legend(loc='upper left')
    
	helpers.save(plt, epos.plotdir+fname+'_x')
	#print epos.plotdir+fname+'_x'

def oneD_y(epos, PlotZoom=False, MCMC=False, PlotQ=False, Occ=False, Convert=False):
	if Occ:
		fname= 'occurrence/posterior' if MCMC else 'occurrence/input'
		xbin= epos.xzoom
		title= 'Planet occurrence ({:.2g}-{:.0f} days)'.format(*epos.xzoom)

	else:	
		fname= 'mcmc/posterior' if MCMC else 'input/parametric_initial'
		xbin=None
		title= 'Marginalized Distribution ({:.2g}-{:.0f} days)'.format(*epos.xtrim)
	
	# initial guess
	pps, _, _, pdf0_Y= periodradius(epos, Init=True, xbin=xbin, Convert=Convert)
	if MCMC:
		# best-fit parameters
		pps, _, _, pdf_Y= periodradius(epos, xbin=xbin, Convert=Convert)
	elif Convert:
		pps_in, _, _, pdf0_Y_in= periodradius(epos, Init=True, xbin=xbin, Convert=False)
	
	''' construct the posterior parameters '''
	if MCMC: plotsample= epos.samples[np.random.randint(len(epos.samples), size=100)]
	
	''' Planet Radius, Mass, or q '''
	M_to_q= 1./(epos.Mstar*cgs.Msun/cgs.Mearth)
	# TODO: zip into previous block
	f, ax = plt.subplots()
	ax.set_title(title)
	if PlotQ:
		ax.set_xlabel(r'Planet Mass Ratio')
		ax.set_ylabel('Occurrence / {}q dloga'.format(epos.plotpars['area']))
		area= 2./3.* np.log10(epos.MC_xvar[-1]/epos.MC_xvar[0])
		yscale= 1. /area
		ytrim= epos.in_ytrim
		yvar= epos.in_yvar * M_to_q
	elif epos.MassRadius or (epos.Msini and not Convert):	
		ax.set_xlabel(r'Planet Mass [M$_\bigoplus$]')
		ax.set_ylabel('Occurrence / {}M'.format(epos.plotpars['area']))
		yscale= 1.
		ytrim= epos.in_ytrim
		yvar= epos.in_yvar
	elif epos.RV:
		ax.set_xlabel(r'Planet Minimum Mass [M$_\bigoplus$]')
		ax.set_ylabel('Occurrence / {}Msini'.format(epos.plotpars['area']))
		yscale= 1.
		ytrim= epos.ytrim
		yvar= epos.MC_yvar
	else:
		ax.set_xlabel(r'Planet Radius [R$_\bigoplus$]')
		ax.set_ylabel('Occurrence / {}R'.format(epos.plotpars['area']))
		yscale= 1.
		ytrim= epos.ytrim
		yvar= epos.MC_yvar
	
	ax.set_xscale('log')
	ax.set_yscale('log')
	ax.set_xlim(np.array(ytrim) * (M_to_q if PlotQ else 1.))
	if 'occrange' in epos.plotpars:
		ax.set_ylim(epos.plotpars['occrange'])
	else:	
		ax.set_ylim([1e-3,1e1])
	
	if MCMC:
		for fpara in plotsample:
			_, _, _, ypdf= periodradius(epos, fpara=fpara, xbin=xbin, Convert=Convert)
			ax.plot(yvar, ypdf*yscale, color='b', alpha=0.1)
		ax.plot(yvar, pdf0_Y*yscale, marker='',ls=':',color='k', label='starting guess')
		ax.plot(yvar, pdf_Y*yscale, marker='',ls='-',color='k', label='best-fit')
	else:
		if 'R break' in epos.fitpars.keys2d:
			ax.axvline(epos.fitpars.get('R break', Init=True), ls='-', color='gray')

		ax.plot(yvar, pdf0_Y*yscale, marker='',ls='-',color='k', label='M sin i' if Convert else None)
		if Convert:
			ax.plot(epos.in_yvar, pdf0_Y_in*yscale, marker='',ls='--',color='k', label='Intrinsic')

	# plot posterior excluding low detection regions (arbitrary 3000 planets assumed)
	#if not (epos.RV or epos.MassRadius):
	if False:
		cens= np.where(epos.f_det<1./3000.,0,1.)
		_, _, _, cens_pdf_Y= periodradius(epos, xbin=xbin, fdet=cens)
		ax.plot(yvar, cens_pdf_Y*yscale, marker='',ls='-',color='green', label='biased')


	if epos.Zoom and not epos.MassRadius: 
		for zoom in epos.yzoom: ax.axvline(zoom, ls='--', color='k')
	
	if PlotQ and 'q Suzuki' in epos.plotpars:
		A, qbr, p1, p2= epos.plotpars['q Suzuki']
		ax.plot(yvar, A*(yvar/qbr)**np.where(yvar<qbr,p1,p2), color='g', \
			label='Suzuki+ 2016' )
		
		#ax.legend(loc='lower right', shadow=False, prop={'size':14}, numpoints=1)
	
	if Occ:
		occbin= epos.occurrence['xzoom']
		ax.errorbar(occbin['yc'], occbin['occ']/occbin['dlny'], 
			yerr= occbin['err']/occbin['dlny'], color='r', marker='_', ls='', capsize=3)
			
	# 	for y, occ, err in zip(occbin['yc'], occbin['occ']/occbin['dlny'], 
	# 						occbin['err']/occbin['dlny']):
	# 		print '{:.3g} {:.3g} {:.3g}'.format(y, occ, err)

	#if Occ or MCMC:
	if MCMC or (Occ and Convert):
		ax.legend(loc='upper right')

	helpers.save(plt, epos.plotdir+fname+('_q' if PlotQ else '_y')+('_convert' if Convert else''))

def twoD(epos, PlotZoom=False, MCMC=False, Convert=False):
	
	# where does this go -> run.py
	assert epos.Parametric
	if not epos.Range: epos.set_ranges()

	# pdf
	pps, pdf, _, _= periodradius(epos, Init= not MCMC, Convert=Convert)
	pdflog= np.log10(pdf) # in %
		
	f, (ax, axb) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[20, 1]})
	f.subplots_adjust(wspace=0)
	
	ax.set_title('Occurrence [%] / d ln p d ln '+('M' if (epos.MassRadius or epos.RV) else 'R'))
	helpers.set_axes(ax, epos, Trim=True, In= (epos.MassRadius or epos.RV) and not Convert)

	''' color scale? '''
	cmap='jet'
	vmin, vmax= -5, 0
	ticks=np.linspace(vmin, vmax, (vmax-vmin)+1)
	levels= np.linspace(vmin, vmax)
	if Convert:
		ax.contourf(epos.X, epos.Y, pdflog, cmap=cmap, levels=levels)
	else:
		ax.contourf(epos.X_in, epos.Y_in, pdflog, cmap=cmap, levels=levels)
	
	# colorbar?
	norm = Normalize(vmin=vmin, vmax=vmax)
	cb1 = clrbar.ColorbarBase(axb, cmap=cmap, norm=norm, ticks=ticks,
                                orientation='vertical') # horizontal
	axb.set_yticklabels(100*10.**ticks)
	axb.tick_params(axis='y', direction='out')
	
	fname= 'mcmc/posterior' if MCMC else 'input/parametric_initial'
	helpers.save(plt, epos.plotdir+fname+('_convert' if Convert else''))

def panels(epos, PlotZoom=False, MCMC=False, Convert=False):
	''' Initial distribution, panel layout'''
	f, (ax, axb, axR, axP)= helpers.make_panels_clrbar(plt)

	# pdf
	pps, pdf, pdf_X, pdf_Y= periodradius(epos, Init= not MCMC, Convert=Convert)
	pdflog= np.log10(pdf) # in %
		
	ax.set_title('Planet Occurrence / dlnP dln'+('M' if epos.MassRadius else 'R'))
	helpers.set_axes(ax, epos, Trim=True, In= (epos.MassRadius or epos.RV) and not Convert)

	# Side panels
	axP.plot(epos.MC_xvar, pdf_X, marker='',ls='-',color='k')
	axR.plot(pdf_Y, epos.MC_yvar if Convert else epos.in_yvar, marker='',ls='-',color='k')

	#helpers.set_axis_distance(axP, epos, Trim=True)
	#helpers.set_axis_size(axR, epos, Trim=True, In= epos.MassRadius)
	axP.set_xlabel(ax.get_xlabel())
	axR.set_ylabel(ax.get_ylabel())

	axP.set_yscale('log')
	axP.set_ylim([2e-3,5])	
	axP.set_yticks([0.01,0.1,1])
	axP.set_yticklabels(['1%','10%','100%'])
	axP.yaxis.tick_right()
	axP.yaxis.set_ticks_position('both')
	axP.tick_params(axis='y', which='minor',left=False,right=False)

	axR.set_xscale('log')
	axR.set_xlim([2e-3,5])
	axR.set_xticks([0.01,0.1,1])
	axR.set_xticklabels(['1%','10%','100%'], rotation=70)
	axR.tick_params(axis='x', which='minor',top=False,bottom=False)
	axP.tick_params(axis='y', which='minor',left=False,right=False)
	axP.tick_params(axis='y', which='major',left=False,right=True)

	''' color scale? '''
	cmap='jet'
	vmin, vmax= -5, 0
	ticks=np.linspace(vmin, vmax, (vmax-vmin)+1)
	levels= np.linspace(vmin, vmax)
	if Convert:
		ax.contourf(epos.X, epos.Y, pdflog, cmap=cmap, levels=levels)
	else:
		ax.contourf(epos.X_in, epos.Y_in, pdflog, cmap=cmap, levels=levels)
	
	# colorbar?
	norm = Normalize(vmin=vmin, vmax=vmax)
	cb1 = clrbar.ColorbarBase(axb, cmap=cmap, norm=norm, ticks=ticks,
                                orientation='vertical') # horizontal
	axb.set_yticklabels(100*10.**ticks)
	axb.tick_params(axis='y', direction='out')
	axb.set_title('%')
	
	helpers.save(plt, epos.plotdir+'input/panels'+('_convert' if Convert else''))



#! /usr/bin/env ipython
''' 
Example script fitting a double broken power-law in period and radius to Kepler data
'''
import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='example_1')

''' load the kepler dr25 exoplanets and survey efficiency '''
obs, survey= EPOS.kepler.dr25(Huber=True, Vetting=True, score=0.9)
epos.set_observation(**obs)
epos.set_survey(**survey)

''' Define a double broken power-law as a planet population '''
epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D)

''' Parameter initial guess and fitting ranges
Note: 
	- brokenpowerlaw2D uses 6 parameters, indicated with the is2D keyword
	- 'pps' is a normalization factor for the planet occurrence rate (planets-per-star)
	- parameter b_R is not fitted
	- dx is the range in walker initial positions for parameters that change sign (+/-)
'''
epos.fitpars.add('pps',		2.0, 	min=0, 			isnorm=True)
epos.fitpars.add('P break',	10.,	min=2,	max=50,	is2D=True)
epos.fitpars.add('a_P',		1.5, 	min=0,			is2D=True)
epos.fitpars.add('b_P',		0.0,	dx=0.1,			is2D=True)
epos.fitpars.add('R break',	3.0,	min=1.0,max=5, 	is2D=True) 
epos.fitpars.add('a_R',		0.0,	dx=0.1, 		is2D=True)
epos.fitpars.add('b_R',		-4.,	fixed=True, 	is2D=True)

''' define the simulated range (trim) and the range compared to observations (zoom)'''
epos.set_ranges(xtrim=[0,730],ytrim=[0.3,20.],xzoom=[2,400],yzoom=[1,6], Occ=True)

''' Run the Monte Carlo Simulation once '''
EPOS.run.once(epos)

''' run an MCMC chain on multiple cores or read in a previously saved run'''
EPOS.run.mcmc(epos, nMC=1000, nwalkers=100, nburn=200, threads=20, Saved=True)

''' define bins where occurrence is calculated'''
epos.set_bins(xbins=[[2,400],[0.9*365,2.2*365]], ybins=[[1,6],[0.7,1.5]]) # eta_zoom, eta_earth

''' Calculate the occurrence rates '''
EPOS.occurrence.all(epos)

''' Adjust axes'''
epos.plotpars['textsize']= 12
epos.xtrim[1]= 1000

''' plot everything '''
EPOS.plot.survey.all(epos)
EPOS.plot.input.all(epos)
EPOS.plot.output.all(epos)
EPOS.plot.mcmc.all(epos)
EPOS.plot.occurrence.all(epos)


#! /usr/bin/env ipython
''' 
This script is meant to reproduce some of the results/figures from
Mulders et al. 2018 (AJ accepted 14 May 2018), results from multi-planet mode:
- Table 2
- Figure 17, 18, 24
- (Figures similar to 6, 7, 8, 9, 11, 12, 13 can also be generated)

Text output will appear in the terminal.
Plots will be generated in the png/ subfolder 

Note that results may vary depending on the random seed, version of the code used, 
and external dependencies. 
Future versions of the code may employ different default parameters.
This script is compatible with version 1.0 of EPOS and was added to the github repository May 15 2018

If you have trouble running this script or EPOS, please consult the online documentation, 
try running the test and example scripts, or contact the first author
'''
import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='mulders2018_multi')

''' load the kepler dr25 exoplanets and survey efficiency '''
obs, survey= EPOS.kepler.dr25(Huber=True, Vetting=True, score=0.9)
epos.set_observation(**obs)
epos.set_survey(**survey)

''' Define a double broken power-law as a planet population '''
epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D)

''' Parameter initial guess and fitting ranges '''
epos.fitpars.add('pps',		0.4, 	min=0, 			isnorm=True)
epos.fitpars.add('P break',	10.,	min=2,	max=50,	is2D=True)
epos.fitpars.add('a_P',		1.5, 	min=0,			is2D=True)
epos.fitpars.add('b_P',		-1,		max=1,	dx=0.1,	is2D=True)
epos.fitpars.add('R break',	3.3,	fixed=True, 	is2D=True) 
epos.fitpars.add('a_R',		-0.5,	fixed=True, 	is2D=True)
epos.fitpars.add('b_R',		-6.,	fixed=True, 	is2D=True)

''' Set parameters for the spacing between planets'''
epos.set_multi(spacing='dimensionless')
epos.fitpars.add('npl', 10, fixed=True) # planets per system
epos.fitpars.add('log D', -0.3)
epos.fitpars.add('sigma', 0.2, min=0)

''' Some additional fit parameters'''
epos.fitpars.add('dR', 0.01, fixed=True)	# Dispersion in planet radii
epos.fitpars.add('inc', 2.0)				# mode of mutual inclinations
epos.fitpars.add('f_iso', 0.4)				# Fraction of isotropic systems
epos.fitpars.add('f_cor', 0.5, fixed=True)	# Correlated noise

''' define the simulated range (trim) and the range compared to observations (zoom)'''
epos.set_ranges(xtrim=[0,730],ytrim=[0.3,20.],xzoom=[2,400],yzoom=[0.5,6])

''' Run the Monte Carlo Simulation once '''
EPOS.run.once(epos)

''' run an MCMC chain on multiple cores or read in a previously saved run'''
EPOS.run.mcmc(epos, nMC=2000, nwalkers=100, nburn=200, threads= 30)

''' Change the titles of the corner plots '''
epos.fitpars.latexkeys={
	'pps':'$\eta_s$', 
	'P break':'$P_{\\rm break}$', 'a_P':'$a_P$', 'b_P':'$b_P$',
	'R break':'$R_{\\rm break}$', 'a_R':'$a_R$', 'b_R':'$b_R$',
	'npl':'$n_p$', 'log D':'log $D$', 'sigma':'$\sigma$',
	'inc':'$i$', 'f_iso':'$f_{\\rm iso}$', 'f_cor':'$f_{\\rm cor}$'
}

''' plot everything '''
EPOS.plot.survey.all(epos)
EPOS.plot.input.all(epos)
EPOS.plot.output.all(epos)
EPOS.plot.mcmc.all(epos)
EPOS.plot.occurrence.all(epos)


import numpy as np
import matplotlib.pyplot as plt
import periodradius, massradius, multi, helpers

clrs= ['r','g','b','m'] # in epos.prep
fmt_symbol= {'ls':'', 'marker':'o', 'mew':2, 'ms':8,'alpha':0.6}

''' output '''
def all(epos, color='C1'):

	if hasattr(epos, 'synthetic_survey'):
		print '\nPlotting output...'
	
		if epos.MonteCarlo:
			periodradius.periodradius(epos, SNR=False)
			periodradius.periodradius(epos, SNR=True)
			if not epos.Parametric:
				periodradius.periodradius(epos, Model=True, SNR=False, color=color)
		else:
			# plot period-radius for non-MC
			pass
		
		periodradius.cdf(epos, color=color)
		periodradius.panels(epos, color=color)
	
		if epos.Multi:
			multi.periodradius(epos)
			multi.periodradius(epos, Nth=True)
	
			multi.multiplicity(epos, MC=True, color=color)
			multi.multiplicity(epos, MC=True, Planets=True, color=color)
			multi.multiplicity_cdf(epos, MC=True)
			multi.periodratio(epos, MC=True, color=color)
			if epos.Parametric and epos.spacing is not None:
				multi.periodratio(epos, MC=True, Input=True, color=color)
			multi.periodratio_cdf(epos, MC=True, color=color)
			multi.periodinner(epos, MC=True, color=color)
			if epos.Parametric and epos.spacing is not None:
				multi.periodinner(epos, MC=True, Input=True, color=color)
			multi.periodinner_cdf(epos, MC=True, color=color)
			# pdf per subgroup
			#periodradius.pdf(epos)
			#periodradius.pdf_3d(epos)

		if epos.MassRadius:
			massradius.massradius(epos, MC=True, color=color)
			massradius.massradius(epos, MC=True, Log=True, color=color)

		else:
			if epos.Parametric and epos.Multi and not epos.RV:
				multi.periodratio(epos, MC=True, N=True)
				if epos.spacing is not None:
					multi.periodratio(epos, MC=True, N=True, Input=True)
				multi.periodinner(epos, MC=True, N=True)

	else:
		print '\nNo output to plot, did you run EPOS.run.once()? \n'



import numpy as np
import os.path
import EPOS

fpath= os.path.dirname(EPOS.__file__)

'''
This module contains helper functions to load RV survey data into EPOS:'
'''

def Mayor2011():
	Msini, P, Completeness = np.loadtxt(fpath+'/files/mayor_completeness.txt', 
		usecols = (0,1,2), unpack = True)
	P_obs, Msini_obs = np.loadtxt(fpath+'/files/Mayor2011.csv', usecols=(2,5), 
			unpack = True, skiprows = 1, delimiter = ',')
	ID = np.loadtxt(fpath+'/files/Mayor2011.csv', usecols = (1,), skiprows = 1, 
			dtype = str, delimiter = ',')
	
	P_1D= np.unique(P)
	Msini_1D= np.unique(Msini)

	X, Y = np.meshgrid(P_1D, Msini_1D)
	Z= Completeness.reshape(X.shape)
	
	obs= {'xvar':P_obs, 'yvar':Msini_obs, 'starID':ID}
	
	survey= {'xvar':P_1D, 'yvar':Msini_1D, 'eff_2D':Z.T}
	obs['nstars']= 822
	
	return obs, survey

def _Howard2010():
	# this function is superseded by the Mayor2011 completeness
	Msini,P,completeness = \
		np.loadtxt('files/msini_per_completeness.txt',usecols=(1,0,2),unpack=True)
	P_obs, Msini_obs = \
		np.loadtxt('files/planet_star_per_msini_ref.txt',usecols=(2,3),unpack=True)
	ID = np.loadtxt('files/planet_star_per_msini_ref.txt',usecols=(1,),dtype=str)

	''' make 2D grid '''
	P_1D= np.unique(P)
	Msini_1D= np.unique(Msini)

	X, Y = np.meshgrid(P_1D, Msini_1D)
	Z= completeness.reshape(X.shape)
	
	obs= {'xvar':P_obs,
		'yvar':Msini_obs, 
		'starID':ID}
	
	survey= {'xvar':P_1D, 'yvar':Msini_1D, 'eff_2D':Z.T} # 'Mstar':None, 'Rstar':None}
	obs['nstars']= 125 # 
	
	return obs, survey


__shortversion__= u'2.0'
__version__= u'2.0.0' # [N!]N(.N)*[{a|b|rc}N][.postN][.devN]

__all__ = ['epos','fitparameters','kepler','rv','run','population','plot','occurrence',
	'fitfunctions','pfmodel','massradius','regression','multi','analytics','save',
	'scripts', 'cgs']

import kepler, rv, run, plot, occurrence, population
import fitfunctions, pfmodel, regression, massradius, multi, analytics, save
import scripts, cgs

from classes import epos, fitparameters


''' Calculate some analytics'''
import numpy as np
import scipy.stats

def pfmodel(epos, sma_cut=1., Verbose=False):
	assert hasattr(epos, 'pfm')
	pfm= epos.pfm
	
	#func= scipy.stats.mstats.gmean
	func=np.median
	
	inc= func(pfm['inc'])
	mass= func(pfm['M'])
	Pin= func(pfm['Pin'])
	dP= func(pfm['dP'][pfm['dP']>0])
	
	_ , counts= np.unique(epos.pfm['ID'],return_counts=True)
	Nk= np.mean(counts)
	
	# How many systems fit in this range with kepler spacing?
	dP_Kep= 2.0 # kepler spacing
	Nobs= np.log10(365.24/Pin) / np.log10(dP_Kep)
	
	s_inc= 'inc= {:.2g} degree'.format(inc)
	s_mass= 'mass= {:.2g} M_earth'.format(mass)
	s_inner= 'inner= {:.2g} days'.format(Pin)
	s_ratio= 'ratio= {:.2g}'.format(dP)
	s_multi= 'Nk= {}'.format(Nk)
	#s_multi= 'Nk= {} ({:.2g})'.format(Nk, Nobs)
	
	s_tab= '${:.2g}$ '.format(mass)
	s_tab+= '& ${:.2g}$ '.format(inc)
	s_tab+= '& ${:.2g}$ '.format(Pin) 
	s_tab+= '& ${:.2g}$ '.format(dP)
	s_tab+= '& ${:.1f}$ '.format(Nk)
	#s_tab+= '& ${:.1f} ~({:.1f})$ '.format(Nk, Nobs)
	s_tab+= r'\\'
	
	if Verbose:
		print s_inc, s_mass, s_inner, s_ratio, s_multi
	
	#return s_inc, s_mass, s_inner, s_ratio, s_multi
	return s_tab

def bic_loglike(log_likelihood, k_free, n_data):
	# natural log
	return -2.* log_likelihood + k_free* np.log(n_data)

def aic_loglike(log_likelihood, k_free):	
	# natural log
	return 2.* k_free -2.* log_likelihood

def aic_c_loglike(log_likelihood, k_free, n_data):	
	# natural log
	aic= aic_loglike(log_likelihood, k_free)
	cfactor= (2.* k_free**2. + 2.*k_free) / (n_data- k_free - 1.)
	return  aic * cfactor

def bic_rss(rss, k_free, n_data):	
	return n_data*np.log(rss/n_data) + k_free *np.log(n_data)

def aic_rss(rss, k_free, n_data):	
	return 2.* k_free + n_data*np.log(rss) # + some constant?

def aic_c_rss(rss, k_free, n_data):	
	aic= aic_rss(rss, k_free, n_data)
	cfactor= (2.* k_free**2. + 2.*k_free) / (n_data- k_free - 1.)
	return  aic * cfactor



import warnings
import numpy as np
import matplotlib.pyplot as plt

import helpers
import parametric, periodradius, multi

try:
	import corner
except ImportError:
	print '\nWarning: corner.py not imported'
	warnings.warn('corner.py not imported',ImportWarning)
except:
	print '??'

def all(epos):
	if hasattr(epos, 'chain'):
		print '\nPlotting chain...'
		chain(epos)
		try:
			corners(epos)
		except NameError:
			print '  (skipping corner plot)'
		
		if epos.Parametric:
			parametric.oneD(epos, MCMC=True)
			parametric.twoD(epos, MCMC=True)
			if epos.Multi:
				parametric.oneD_x(epos, MCMC=True, Log=False)
				
		
		# plot sample from posterior
		if hasattr(epos, 'plotsample'):
			periodradius.panels(epos, MCMC=True)
			if epos.Multi:
				multi.multiplicity(epos, MCMC=True)
				multi.periodratio(epos, MCMC=True)
				multi.periodinner(epos, MCMC=True)
			
	else:
		print '\nNo chain to plot, did you run EPOS.run.mcmc()? \n'
	
def chain(epos):
	nwalker, nstep, npara= epos.chain.shape

	# axes grid
	f, axlist = plt.subplots((npara+1)/2, 2, sharex=True)
	f.set_size_inches(8,8)

	#axlis.set_title('')
	for axl in axlist[-1]:
		axl.set_xlabel('Step number')
	
	# labels
	if hasattr(epos.fitpars,'latexkeys'):
		labels= [epos.fitpars.latexkeys[key] if key in epos.fitpars.latexkeys else key for key in epos.fitpars.keysfit]
	else:
		labels=epos.fitpars.keysfit

	# loop over all axes
	for k, ax in enumerate(axlist.flatten(order='F')):
		
		try:
			ax.set_ylabel(labels[k])
			#ax.set_xlim(1, 10)
	
			#plot each walker
			for i in range(nwalker):
				ax.plot(epos.chain[i,:,k],color='r',alpha=10./nwalker)
	
			ax.axvline(epos.burnin, ls='--', color='k')
		except : 	
			ax.set_visible(False)

	
	f.subplots_adjust(hspace=0.15, wspace=0.4)
			
	helpers.save(plt, '{}mcmc/chain'.format(epos.plotdir))	

def corners(epos):
	if hasattr(epos.fitpars,'latexkeys'):
		labels= [epos.fitpars.latexkeys[key] if key in epos.fitpars.latexkeys else key for key in epos.fitpars.keysfit]
	else:
		labels=epos.fitpars.keysfit
		
	fig = corner.corner(epos.samples, labels=labels,
                      truths=epos.fitpars.getfit(Init=True), 
                      quantiles=[0.16, 0.5, 0.84], show_titles=True)
	fig.savefig('{}mcmc/triangle.png'.format(epos.plotdir))
		



#! /usr/bin/env ipython
''' 
Test if EPOS can read exoplanet data in files/

Plots should appear in the directory
png/test_1/survey/
'''

import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='test_1')

''' load the kepler dr25 exoplanets and survey efficiency '''
obs, survey= EPOS.kepler.dr25(Huber=True, Vetting=True, score=0.9)
epos.set_observation(**obs)
epos.set_survey(**survey)

''' plot survey '''
EPOS.plot.survey.all(epos)



'''
simple regression helper function
'''
import numpy as np

# simple sliding window, width is factor width (2)
def sliding_window_log(x, y, xgrid, width=2.):
	if y is None: y=np.ones_like(x)
	f= np.zeros_like(xgrid)
	for i, (_x, _y) in enumerate(zip(x,y)):
		#f+= np.where( 
		f[np.abs(np.log10(xgrid/_x))< (np.log10(width)/2.) ]+= _y
	return f
		


#import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import matplotlib.colorbar as clrbar
import matplotlib.colors
import numpy as np

import helpers, parametric
from EPOS.population import periodradius

clrs= ['r','g','b','m'] # in epos.prep
#fmt_symbol= {'ls':'', 'marker':'o', 'mew':2, 'ms':8,'alpha':0.6}

def all(epos, color=None, alpha_fac=None):
	assert epos.Observation
	if hasattr(epos, 'occurrence'):
		
		if 'planet' in epos.occurrence:
			colored(epos)

		if 'model' in epos.occurrence:
			model(epos, color=color)
			if alpha_fac is not None:
				model(epos, color=color, alpha_fac=alpha_fac)
			#if Fade:
			model(epos, color=color, Gradient=True)

		if 'poly' in epos.occurrence:
			colored(epos, Poly=True)
			if 'model' in epos.occurrence:
				model(epos, color=color, alpha_fac=alpha_fac, Poly=True)
				if 'labels' in epos.occurrence['poly']:
					# only callable with models right now
					poly_only(epos)

		
		if 'bin' in epos.occurrence:
			colored(epos, Bins=True)
			if 'model' in epos.occurrence:
				model(epos, color=color, alpha_fac=alpha_fac, Bins=True)
	
			if 'eta0' in epos.occurrence['bin']:
				integrated(epos)
				if 'eta' in epos.occurrence['bin']:
					integrated(epos, MCMC=True)
					integrated(epos, MCMC=True,Planets=True)

		if 'xzoom' in epos.occurrence:
			if epos.Parametric:
				parametric.oneD(epos, Occ=True)
				if not epos.MonteCarlo and epos.Msini:
					parametric.oneD_y(epos, Occ=True, Convert=True)

				if hasattr(epos, 'chain'):
					 parametric.oneD(epos, Occ=True, MCMC=True)
					 if epos.Msini:
					 	parametric.oneD_y(epos, Occ=True, MCMC=True, Convert=True)
	else:
		print '\nNo occurrence to plot, did you run EPOS.occurrence.all()? \n'
	
def colored(epos, Bins=False, Poly=False):
	
	f, (ax, axb) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[20, 1]})
	f.subplots_adjust(wspace=0)
	
	name= 'Survey Completeness'
	if epos.name in ['dr25_F','dr25_G','dr25_K','dr25_M','dr25_GK']: name+= ' ('+epos.name[5:]+')'
	ax.set_title(name)
	
	helpers.set_axes(ax, epos, Trim=True)
	#if epos.plot['']
	
	#ax.plot(epos.obs_xvar, epos.obs_yvar, ls='', marker='.', mew=0, ms=5.0, color='k')

	''' color scale? '''
	cmap='magma' # viridis, plasma, inferno, magma, spring, cool
	cmap='viridis'
	vmin, vmax= -4, 0
	ticks=np.linspace(vmin, vmax, (vmax-vmin)+1)
	clrs, norm= helpers.color_array(np.log10(epos.occurrence['planet']['completeness']),
		vmin=vmin,vmax=vmax, cmap=cmap)
	ax.scatter(epos.obs_xvar, epos.obs_yvar, color=clrs, s=4)
	
	# colorbar?
	cb1 = clrbar.ColorbarBase(axb, cmap=cmap, norm=norm, ticks=ticks,
                                orientation='vertical') # horizontal
	axb.set_yticklabels(100*10.**ticks)
	axb.tick_params(axis='y', direction='out')
	
	''' bins?'''
	if Bins:
		occbin= epos.occurrence['bin']
		for k, (xbin, ybin, n, inbin, occ) in enumerate(
				zip(occbin['x'],occbin['y'],occbin['n'],occbin['i'], occbin['occ'])
				):
			clr= clrs[k%4]
		
			# colored dots
			#ax.plot(epos.obs_xvar[inbin], epos.obs_yvar[inbin], 
			#	ls='', marker='.', mew=0, ms=5.0, color=clr, zorder=1)
		
			# box
			ax.add_patch(patches.Rectangle( (xbin[0],ybin[0]), 
				xbin[1]-xbin[0], ybin[1]-ybin[0],
				fill=False, zorder=2, ls='-', color='k') )
		
			xnudge=1.01
			ynudge=1.02
			 
			size=16 if not 'textsize' in epos.plotpars else epos.plotpars['textsize'] 
				# 12 fit in box, 16 default
			ax.text(xbin[0]*xnudge,ybin[1]/ynudge,'{:.1%}'.format(occ), va='top', 
				size=size)
			ax.text(xbin[0]*xnudge,ybin[1]/ynudge,'\n$\pm${:.1f}'.format(
				occbin['err'][k]*100), va='top', size=size)
			ax.text(xbin[1]/xnudge,ybin[0]*ynudge,'n={}'.format(n), ha='right',
				size=size)
	
		helpers.save(plt, epos.plotdir+'occurrence/bins')
	elif Poly:
		occpoly= epos.occurrence['poly']
		for k, (xc, yc, coords, n, inbin, occ, err) in enumerate(
				zip(occpoly['xc'],occpoly['yc'],occpoly['coords'],
					occpoly['n'],occpoly['i'], occpoly['occ'], occpoly['err'])
				):
				
			# box
			ax.add_patch(matplotlib.patches.Polygon(coords,
				fill=False, zorder=2, ls='-', color='k') )
			 
			size=16 if not 'textsize' in epos.plotpars else epos.plotpars['textsize'] 
				# 12 fit in box, 16 default
			ax.text(xc,yc,'{:.1%}\n$\pm${:.1f}'.format(occ, err), ha='center', va='center', 
				size=size)
			#ax.text(xbin[1]/xnudge,ybin[0]*ynudge,'n={}'.format(n), ha='right',
			#	size=size)
	
		helpers.save(plt, epos.plotdir+'occurrence/poly')
	else:
		helpers.save(plt, epos.plotdir+'occurrence/colored')
		
def integrated(epos, MCMC=False, Planets=False):
	
	f, (ax, axb) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[20, 1]})
	f.subplots_adjust(wspace=0)
	
	sy= 'M' if (epos.MassRadius or epos.RV) else 'R'
	ax.set_title('Occurrence'+ (' (dln'+sy+' dlnP)' if MCMC else ' (Initial Guess)'))
	
	helpers.set_axes(ax, epos, Trim=True, In=epos.MassRadius)
	
	''' color scale? '''
	cmap='jet' # cool, spring
	vmin, vmax= -5, 0
	ticks=np.linspace(vmin, vmax, (vmax-vmin)+1)
	levels=np.linspace(vmin, vmax, 256)
	
	''' 2D pdf '''
	pps, pdf, _, _= periodradius(epos, Init=not MCMC)
	pdflog= np.log10(pdf) # in %
	cs= ax.contourf(epos.X_in, epos.Y_in, pdflog, cmap=cmap, levels=levels)
	cbar= f.colorbar(cs, cax=axb, ticks=ticks)
	axb.set_yticklabels(100*10.**ticks)
	axb.tick_params(axis='y', direction='out')
	axb.set_title('%')
	
	''' integrated occurrence per bin'''
	occbin= epos.occurrence['bin']
	key = 'eta' if MCMC else 'eta0'
	for k, (xbin, ybin, n, inbin, occ) in enumerate(
			zip(occbin['x'],occbin['y in'],occbin['n'],occbin['i'], occbin[key])
			):
		clr= clrs[k%4]
	
		# colored dots
		#ax.plot(epos.obs_xvar[inbin], epos.obs_yvar[inbin], 
		#	ls='', marker='.', mew=0, ms=5.0, color=clr, zorder=1)
	
		# box
		ax.add_patch(patches.Rectangle( (xbin[0],ybin[0]), 
			xbin[1]-xbin[0], ybin[1]-ybin[0],
			fill=False, zorder=2, ls='-', color='k') )
	
		xnudge=1.01
		ynudge=1.02
		size=16 if not 'textsize' in epos.plotpars else epos.plotpars['textsize'] 
				# 12 fit in box, 16 default
		ax.text(xbin[0]*xnudge,ybin[1]/ynudge,'{:.1%}'.format(occ), va='top',size=size)
		if MCMC:
			ax.text(xbin[0]*xnudge,ybin[1]/ynudge,'\n +{:.1%}\n  -{:.1%}'.format(
				occbin['eta+'][k],occbin['eta-'][k]
				), va='top',size=size)

	''' overplot planets '''
	if Planets:
		ax.plot(epos.obs_xvar, epos.obs_yvar, 
			ls='', marker='.', mew=0, ms=5, alpha=1, color='k')

	fname= 'posterior' if MCMC else 'integrated'
	if Planets: fname+= '.planets'
	helpers.save(plt, epos.plotdir+'occurrence/'+fname)

def model(epos, color='C0', alpha_fac=None, Bins=False, Poly=False, Gradient=False):
	
	f, ax = plt.subplots()
	
	name= '{}, $\eta={:.2g}$'.format(epos.name, epos.occurrence['model']['eta'])
	ax.set_title(name)
	
	helpers.set_axes(ax, epos, Trim=True)
	
	# set transparency / color gradient
	if Gradient:
		suffix= '.gradient'

		weigths= epos.occurrence['model']['completeness']
		cmin, cmax= 0.001, 0.1

		weigths= np.maximum(np.minimum(weigths,cmax), cmin)

		cmap='copper_r'
		#ticks=np.linspace(vmin, vmax, (vmax-vmin)+1)
		clrs, norm= helpers.color_array(np.log10(weigths),
			vmin=np.log10(cmin),vmax=np.log10(cmax), cmap=cmap)
		
		ax.scatter(epos.pfm['P'], epos.pfm['R'], 
			marker='o', s=13, lw=0, color=clrs,zorder=0)

		# colorbar?
		# cb1 = clrbar.ColorbarBase(axb, cmap=cmap, norm=norm, ticks=ticks,
		#                             orientation='vertical') # horizontal
		# axb.set_yticklabels(100*10.**ticks)
		# axb.tick_params(axis='y', direction='out')


	elif alpha_fac is not None:
		suffix= '.alpha'

		weigths= epos.occurrence['model']['completeness']*alpha_fac #*epos.nstars
		alpha= np.maximum(np.minimum(weigths,1.), 0.0) # 0.2?

		if True:
			# color issues with  to_rgba_array
			clr_rgba = np.empty((len(alpha), 4), float)
			for i, a in enumerate(alpha):
				clr_rgba[i] = matplotlib.colors.to_rgba(color, a)

		else:
			clr= np.full_like(weigths,color,dtype=str)
			clr_rgba= matplotlib.colors.to_rgba_array(clr) # alpha
			#print clr_rgba[0,:]
			clr_rgba[:,3]= alpha

		ax.scatter(epos.pfm['P'], epos.pfm['R'], 
			marker='o', s=13, lw=0, color=clr_rgba,zorder=0)
	else:
		suffix=''
		clr= matplotlib.colors.to_rgba(color)
		ax.plot(epos.pfm['P'], epos.pfm['R'], ls='', marker='o', mew=0, ms=4, 
			color=clr, zorder=0)
	
	''' bins'''
	if Bins:
		occbin= epos.occurrence['model']['bin']
		for k, (xbin, ybin, n, inbin, occ) in enumerate(
				zip(occbin['x'],occbin['y'],occbin['n'],occbin['i'], occbin['occ'])
				):
		
			# box
			ax.add_patch(patches.Rectangle( (xbin[0],ybin[0]), 
				xbin[1]-xbin[0], ybin[1]-ybin[0],
				fill=False, zorder=2, ls='-', color='k') )
		
			xnudge=1.01
			ynudge=1.02
			 
			size=16 if not 'textsize' in epos.plotpars else epos.plotpars['textsize'] 
				# 12 fit in box, 16 default
			ax.text(xbin[0]*xnudge,ybin[1]/ynudge,'{:.1%}'.format(occ), va='top', 
				size=size)
			ax.text(xbin[0]*xnudge,ybin[1]/ynudge,'\n$\pm${:.1f}'.format(
				occbin['err'][k]*100), va='top', size=size)
			ax.text(xbin[1]/xnudge,ybin[0]*ynudge,'n={}'.format(n), ha='right',
				size=size)
		helpers.save(plt, epos.plotdir+'occurrence/model_bins'+suffix)
	elif Poly:
		occpoly= epos.occurrence['model']['poly']
		for k, (xc, yc, coords, n, inbin, occ, err) in enumerate(
				zip(occpoly['xc'],occpoly['yc'],occpoly['coords'],
					occpoly['n'],occpoly['i'], occpoly['occ'], occpoly['err'])
				):
				
			# box
			ax.add_patch(matplotlib.patches.Polygon(coords,
				fill=False, zorder=2, ls='-', color='k') )
			 
			size=16 if not 'textsize' in epos.plotpars else epos.plotpars['textsize'] 
				# 12 fit in box, 16 default
			ax.text(xc,yc,'{:.1%}\n$\pm${:.1%}'.format(occ, err), ha='center', va='center', 
				size=size)
		helpers.save(plt, epos.plotdir+'occurrence/model_poly'+suffix)
	else:
		helpers.save(plt, epos.plotdir+'occurrence/model'+suffix)

def poly_only(epos):
	
	f, ax = plt.subplots()
	
	ax.set_title('Planet Classes')
	
	helpers.set_axes(ax, epos, Trim=True)
	
	# coordinates are from model routine
	occpoly= epos.occurrence['model']['poly']

	for k, (xc, yc, coords, label) in enumerate(
			zip(occpoly['xc'],occpoly['yc'],occpoly['coords'],
				epos.occurrence['poly']['labels'])
			):
			
		# box
		ax.add_patch(matplotlib.patches.Polygon(coords,
			fill=False, zorder=2, ls='-', color='k') )
		 
		size=16 if not 'textsize' in epos.plotpars else epos.plotpars['textsize'] 
			# 12 fit in box, 16 default
		ax.text(xc,yc,label, ha='center', va='center', 
			size=size)

	helpers.save(plt, epos.plotdir+'occurrence/poly_only')




''' 
Save output to files
json/.../occurrence.??.json
'''
import numpy as np
import os, json

def survey(epos, Verbose=False):
	if hasattr(epos,'obs_zoom'):
		if not os.path.isdir(epos.jsondir): os.makedirs(epos.jsondir)

		print '\nStoring Observations'

		if Verbose:
			print '  keys in epos.obs_zoom:'
			print '  ',epos.obs_zoom.keys()

		keys=epos.obs_zoom.keys()
		save_to_json(epos,'obs_zoom',epos.obs_zoom, keys)

		if 'multi' in epos.obs_zoom:
			keys= epos.obs_zoom['multi'].keys()
			if Verbose: print '  keys in multi:'
			print '  ',keys
			save_to_json(epos,'obs_zoom.multi',epos.obs_zoom['multi'], keys)
	else:
		print 'No survey stored, did you run epos.set_survey() and set_observation() ?'

def occurrence(epos, Verbose=False):
	if hasattr(epos,'occurrence'):
		if not os.path.isdir(epos.jsondir): os.makedirs(epos.jsondir)
		
		focc= epos.occurrence
		
		if Verbose:
			print 'keys in epos.occurrence'
			for key in focc:
				print '\n{}'.format(key)
				for subkey in focc[key]:
					print '  {}'.format(subkey)

		if 'model' in epos.occurrence:
			save_to_json(epos,'occurrence.model',focc['model'], ['eta', 'completeness'])

		if 'planet' in epos.occurrence:
			save_to_json(epos,'occurrence.planet',focc['planet'],['xvar','yvar','completeness','occ'])

		if 'bin' in epos.occurrence:
			gridkeys= ['xc', 'yc', 'dlnx', 'dlny', 'y', 'x']
			
			# inverse detection efficiency
			if 'occ' in focc['bin']:
				keys=['occ', 'err', 'n']+gridkeys
				save_to_json(epos, 'occurrence.inverse', focc['bin'], keys)
			
			# initial guess
			if 'eta0' in focc['bin']:
				keys=['eta0','gamma0','area']+gridkeys
				save_to_json(epos, 'occurrence.initial', focc['bin'], keys)
			
			# posterior
			if 'eta' in focc['bin']:
				keys=['eta','eta+','eta-','gamma','gamma+','gamma-','area']+gridkeys
				save_to_json(epos, 'occurrence.posterior', focc['bin'], keys)

		# occurrence along x and y axes
		if 'xzoom' in epos.occurrence and 'yzoom' in epos.occurrence:
			pass
		
	else:
		print 'No occurrence rates found, did run use epos.occurrence.all() ?'

def synthetic_survey(epos, Verbose=False):
	if hasattr(epos,'synthetic_survey'):
		if not os.path.isdir(epos.jsondir): os.makedirs(epos.jsondir)

		ss= epos.synthetic_survey

		print '\nStoring Synthetic Survey'

		if Verbose:
			print '  keys in epos.synthetic_survey:'
			print '  ',ss.keys()

		keys=['P', 'Y', 'M', 'R', 'P zoom', 'Y zoom']
		save_to_json(epos,'synthetic_survey',ss, keys)

		if 'multi' in ss:
			keys= ss['multi'].keys()
			if Verbose: print '  keys in multi:'
			print '  ',keys
			save_to_json(epos,'synthetic_survey.multi',ss['multi'], keys)
	else:
		print 'No synthetic survey stored, did you run epos.run.once() ?'
	
def population(epos, Verbose=False):
	if hasattr(epos,'population'):
		if not os.path.isdir(epos.jsondir): os.makedirs(epos.jsondir)
		
		pop= epos.population

		print '\nStoring Planet Population'

		if Verbose:
			print 'keys in epos.population'
			for key in pop:
				print '\n{}'.format(key)
				for subkey in pop[key]:
					print '  {}'.format(subkey)

		if 'system' in epos.population:
			keys=['P', 'Y', 'ID', 'inc', 'detectable']
			save_to_json(epos,'population.systems',pop['system'], keys)
		
	else:
		print 'No population stored, did you run epos.run.once() ?'

def model(epos, Verbose=False):
	if hasattr(epos,'pfm'):
		if not os.path.isdir(epos.jsondir): os.makedirs(epos.jsondir)

		print '\nStoring Models'

		if Verbose:
			print '  keys in epos.pfm:'
			print '  ',epos.pfm.keys()

		keys=epos.pfm.keys()
		save_to_json(epos,'pfm',epos.pfm, keys)

	else:
		print 'No model stored, did you run epos.set_population() ?'
	
def save_to_json(epos, fname, bigdict, keys):
	tosave= {x: bigdict[x] for x in keys if x in bigdict}
	with open('{}{}.json'.format(epos.jsondir,fname), 'w') as f:
		json.dump(tosave, f, default=serialize_numpy_array)
	
def serialize_numpy_array(obj):
	if isinstance(obj, np.ndarray):
		return obj.tolist()
	else:
		raise TypeError('Not serializable')



import matplotlib.pyplot as plt
import numpy as np
import helpers

clrs= ['r','g','b','m'] # in epos.prep
fmt_symbol= {'ls':'', 'marker':'o', 'mew':2, 'ms':8,'alpha':0.6}

def readme():
	print '\nMakes all the plots'
	print 
	print 'observed:'
	print 'model:'
	print 'result:'


''' parameter study '''
def multiD(epos):
	print '\nPlotting parameter study...'
	
	# identify indices to dimensions with arrays
	k_dim= []
	for k, dim in enumerate(epos.para['grid']): 
		if len(dim)>1: k_dim.append(k) 
	
	all_dim= epos.para['prob'].shape
	ndim= len(k_dim) 
	indices= range(len(all_dim)) 
				
	print '  {} dimensional; {}'.format(ndim, k_dim)
	if ndim>1:
		for dim in k_dim:
			# sum over all axes _except_ dim :S
			index= tuple(indices[x] for x in indices if (x is not dim))
			print dim, index
			p_1d= np.sum(epos.para['prob'], axis=index)
			print p_1d.shape
			para_1d(epos, p_1d, epos.para['grid'][dim], 'dim{}'.format(dim))

def oneD(epos, pdf, xgrid, fname):
	f, ax = plt.subplots()
	
	ax.set_title('Marginalized PDF')
	ax.set_xlabel(fname)
	ax.set_ylabel('Probability')

	ax.plot(xgrid, pdf, ls='-', marker='+', color='b')
	
	aux.save(plt, '{}grid/1d.{}'.format(epos.plotdir,fname))		


'''This module contains some fitting functions for parametric fits to Kepler data
These functions work on numpy arrays ONLY, 
all functions return a non-normalized probability density distribution, 
assuming x and y are numpy arrays of equal length (i.e. coordinates)
'''

import numpy as np
import scipy.stats

def uniform(x,y): 
	''' Uniform distribution'''
	return np.ones_like(x)

def powerlaw2D(x, y, p1, p2):
	''' Power law in x and y'''
	return x**p1 * y**p2

def powerlaw2D_yonly(x, y, p1):
	''' Power law in y, uniform distribution in x'''
	return y**p1

def brokenpowerlaw2D(x, y, xp, p1, p2, yp, p3, p4):
	''' Broken powerlaw in x and y
	
	Args:
		x(np.array): x
		y(np.array): y
		xp(float): break in x
		p1(float): power law index at x<xp
		p2(float): power law index at x>xp
		yp(float): break in y
		p3(float): power law index at y<yp
		p4(float): power law index at y>yp
	'''
	return brokenpowerlaw1D(x, xp, p1, p2) * brokenpowerlaw1D(y, yp, p3, p4)

def brokenpowerlaw2D_xonly(x, y, xp, p1, p2, p3):
	return brokenpowerlaw1D(x, xp, p1, p2) * y**p3

def brokenpowerlaw2D_yonly(x, y, p1, yp, p3, p4):
	return x**p1 * brokenpowerlaw1D(y, yp, p3, p4)

def brokenpowerlaw2D_symmetric(x, y, xp, p1, yp, p3, p4):
	return brokenpowerlaw2D(x, y, xp, p1, -p1, yp, p3, p4)

def doublebrokenpowerlaw2D(x, y, xp, p1, p2, yp, p3, p4, yr, yrp, p5, p6):
	''' Broken powerlaw in x, double broken power-law in y
	
	Args:
		x(np.array): x
		y(np.array): y
		xp(float): break in x
		p1(float): power law index at x<xp
		p2(float): power law index at x>xp
		yp(float): break in y
		p3(float): power law index at y<yp
		p4(float): power law index at y>yp
		yr(float): ratio between power-law normalizations
		yrp(float): break in y
		p5(float): power law index at y<yp
		p6(float): power law index at y>yp		
	'''
	xfunc= brokenpowerlaw1D(x, xp, p1, p2)
	yfunc= brokenpowerlaw1D(y, yp, p3, p4) + yr* brokenpowerlaw1D(y, yrp, p5, p6) 
	return xfunc* yfunc
	
def brokenpowerlaw1D(x, xp, p1, p2):
	# normalized to 1 at x=xp
	return (x/xp)**np.where(x<xp,p1,p2)

def lognormal_size(x, y, xp, p1, p2, y0, dy):
	'''
	Lognormal distribution in planet size `y` and a broken power law in distance `x` 
	
	Args:
		x(np.array): x
		y(np.array): y
		xp(float): break in x
		p1(float): power law index at x<xp
		p2(float): power law index at x>xp
		y(float): mean of y
		dy(flat): dispersion of y, in dex	
	Note:		
		Only works with support for a non-separable function in x and y
	'''	
	return brokenpowerlaw1D(x, xp, p1, p2) * \
			scipy.stats.norm.pdf(np.log10(y), loc=np.log10(y0), scale=dy)

def lognormal_rise(x, y, y0, dy, yp):
	'''
	Lognormal distribution in planet size `y` that rises with distance `x` 
	
	Args:
		x(np.array): x
		y(np.array): y
		y(float): mean of y
		dy(flat): dispersion of y, in dex	
		yp(float): break in x
	Note:		
		Only works with support for a non-separable function in x and y
	'''	
	return scipy.stats.norm.pdf(np.log10(y), loc=np.log10(y0* x**yp), scale=dy)

def schechter1D(x, xp, p1):
	''' Schechter function normalized to p break'''
	return (x/xp)**p1 * np.exp(-x/xp)

def schechter_size(x, y, xp, p1, p2, yp, p3):
	'''
	Schechter distribution in planet size `y` and a broken power law in distance `x` 
	
	Args:
		x(np.array): x
		y(np.array): y
		xp(float): break in x
		p1(float): power law index at x<xp
		p2(float): power law index at x>xp
		y(float): mean of y
		dy(flat): dispersion of y, in dex	
	Note:		
		Only works with support for a non-separable function in x and y
	'''	
	return brokenpowerlaw1D(x, xp, p1, p2) * \
			schechter1D(y,yp,p3)

def bimodal2D(x, y, rp0, rxp, rp1, rp2, ry, ryw, gxp, gp1, gp2, gy, gyw):
	''' Bimodal distribution where each component (r,g) is a broken power law in `x` 
	and a lognormal in `y`
	
	Args:
		x(np.array): x
		y(np.array): y
		rp0(float): ratio of rocky `r` component versus gaseous `g` component
		rxp(float): break in x
		rp1(float): power law index at x<xp
		rp2(float): power law index at x>xp
		ry(float): mean of y
		ryw(flat): dispersion of y, in dex	
	Note:		
		Only works with support for a non-separable function in x and y
	'''
	rocky= rp0 * lognormal_size(x, y, rxp, rp1, rp2, ry, ryw)
	gas=   lognormal_size(x, y, gxp, gp1, gp2, gy, gyw)
	return rocky+gas



#! /usr/bin/env ipython
''' 
Test if EPOS can do an MCMC fit with emcee on multiple cores

Plots should appear in the directory
png/test_3/mcmc/
png/test_3/output/
'''

import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='test_4')

''' load the kepler dr25 exoplanets and survey efficiency '''
obs, survey= EPOS.kepler.dr25(Huber=True, Vetting=True, score=0.9)
epos.set_observation(**obs)
epos.set_survey(**survey)

''' define the parameteric distribution, here a power-law in radius and period '''
epos.set_parametric(EPOS.fitfunctions.powerlaw2D)
epos.fitpars.add('pps', 2.0, min=0)
epos.fitpars.add('P1',0.3, is2D=True)
epos.fitpars.add('P2',-0.2, dx=0.1, is2D=True)

''' define the simulated range (trim) and the range compared to observations (zoom) '''
epos.set_ranges(xtrim=[10,730],ytrim=[0.5,4.],xzoom=[20,300],yzoom=[0.7,3])

''' Run the Monte Carlo Simulation once '''
EPOS.run.once(epos)

''' run a longer MCMC chain on multiple cores '''
EPOS.run.mcmc(epos, nMC=1000, nwalkers=100, nburn=200, threads=8, Saved=False) # ~10 mins

''' plot mcmc chain and simulated output '''
EPOS.plot.mcmc.all(epos)
EPOS.plot.output.all(epos)


'''
his module contains helper functions for multiplanet systems
'''
import numpy as np

def indices(ID, Verbose=False):
	''' 
	retruns indices to planets that are observed to be single or multi
	
	Args:
		ID(np.array):  array of planet host star identifiers. 
			Planets with the same ID are in the same system.
	
	Returns:
		single(np.array of int): index to observed single systems
		multi(np.array of int): index to observed multis systems
		k(list of int): planetary systems witk k planets
		multis(np.array of int): indices to observed systems witk k planets
	
	'''
	IDsys, toplanet, counts= np.unique(ID, return_inverse=True,return_counts=True)
	if Verbose:
		print '\n  {} singles, {} multis'.format(np.sum(counts==1),np.sum(counts>1))
	
	single=(counts==1)[toplanet]
	multi= (counts>1)[toplanet]
	ksys= range(2,len(np.bincount(counts)))
	multis= [(counts==k)[toplanet] for k in ksys] # tuple for indexing
	
	return single, multi, ksys, multis

def nth_planet(ID, P):
	'''
	Index to observed nth planet in the system
	
	Args:
		ID(np.array):  array of planet host star identifiers. 
			Planets with the same ID are in the same system.
		ID(np.array):  array of planet orbital periods. 
	
	Returns:
		single(np.array of int): index to observed single systems
		multi(np.array of int): index to observed multis systems
		nth(list of int): nth planet in the system
		multis(np.array of int): indices to nth planet
	'''
	IDsys, toplanet, counts= np.unique(ID, return_inverse=True,return_counts=True)
	assert np.all(IDsys[toplanet] == ID)

	# get index of first planet in each system
	i1= _first_planet_in_system(counts)	

	single=(counts==1)[toplanet]
	multi= (counts>1)[toplanet]

	ksys= range(1, len(np.bincount(counts))) # [1,2,3,...n]
	#ksys= range(2, len(np.bincount(counts)))
	
	multis=  [i1[counts>1]] # 1st planet in multi
	for i in ksys[1:]:
	#for i in ksys:
		# 2nd, 3rd, ..
		multis.append(tuple([i1[counts>=i]+(i-1)])) # tuple for indexing
		
	return single, multi, ksys, multis

def frequency(ID, Verbose=False):
	'''
	returns the frequency of single/double/triple/etc systems
	
	Args:
		ID(np.array):  array of planet host star identifiers. 
			Planets with the same ID are in the same system.
	'''
	smulti= ['single','double','triple','quad','quint','sext','sept','oct','nint']
	#bc= np.bincount(np.bincount(ID)) # only for int
	_ , counts= np.unique(ID,return_counts=True)
	bincounts= np.bincount(counts)
	assert bincounts[0]==0, "unique items can't have frequency zero"
	if Verbose:
		for nmulti, text in zip(bincounts[1:], smulti):
			print '  - {}: {}'.format(text, nmulti)
	
	return np.arange(1,bincounts.size), bincounts[1:]

def cdf(ID, Verbose=False):
	'''
	returns the cdf of planets in multi-systems
	'''
	bin, count= frequency(ID)
	xlist=[[bin[k]]*count[k]*bin[k] for k in range(len(bin))]
	#if Verbose: print '  multi cdf planets: {}'.format(len(np.concatenate(xlist)))
	return np.concatenate(xlist)

def periodratio(ID, P, N=None, R=None, Verbose=False):
	'''
	returns the period ratios of adjacent planets as a list
	'''
	IDsys, toplanet, counts= np.unique(ID, return_inverse=True,return_counts=True)

	ismulti= (counts>1) # systems that are multi
	Pmulti= P[ismulti[toplanet]] # Periods of _all_ planets in multis
	assert np.all(IDsys[toplanet] == ID)

	# get index of first planet in each system
	i1= _first_planet_in_system(counts)	
	assert np.all(IDsys == ID[i1]), ' assumes lexsort( (P,ID) )'
	Pinner= P[i1[counts>1]] # innerplanet in multi
	
	Pratio= []
	Rpair= [] # size of inner planet in pair
	for i in range(2,len(np.bincount(counts)) ):
		#print '\nmultiplicity: {}'.format(i)
		im= i1[counts>=i] # multis with 
		_dP= P[im+(i-1)]/P[im+(i-2)]
		if R is not None:
			_R= R[im+(i-1)] # size of outer planet
		#for k, _dP in zip(im,Pratio):
		#	print ' ID {}, P={}, dP= {}'.format(ID[k], P[ID==ID[k]], _dP)
		#print 'i={}: {}'.format(i,dP.size)
		Pratio.extend(_dP)
		if R is not None:
			Rpair.extend(_R)
	#print 'n dP= {}'.format(len(Pratio))
	
	if R is not None:
		return np.array(Pratio), Pinner, Rpair
	elif N is None:
		return np.array(Pratio), Pinner
	else:
		''' Innermost observed planet is nth planet'''
		PN, dPN= [], []
		for m in range(1,10):
			PN.append(Pinner[N[i1[counts>1]]==m])
			#print m, np.sum(N[i1[counts>1]]==m) # mostly 1 or 2, never 3+
			#print N[i1[counts>1]]
			
			# Period ratio of adjacent planets?
			#print 'm={}'.format(m)
			PratioN= []
			for i in range(2,len(np.bincount(counts)) ):
				im= i1[counts>=i] # multis with 
				idx= N[im+(i-1)]-N[im+(i-2)] == m # adjacent, 1,2,3... inbetween
				#print N[im+(i-1)]- N[im+(i-2)]
				#print i, im.size, 
				#print m, idx.sum()
				dP= P[im[idx]+(i-1)]/P[im[idx]+(i-2)]
				PratioN.extend(dP)
			#print 'm={}, n dPN={}'.format(m, len(PratioN))
			dPN.append(PratioN)

		#return np.array(Pratio), Pinner, PN, dPN
		return PN, dPN

def _first_planet_in_system(counts):
	#_, counts= np.unique(ID, return_counts=True) # slower
	di= np.roll(counts,1)
	di[0]=0
	return np.cumsum(di) # index to first planet



import numpy as np
from scipy import interpolate
from scipy.stats import ks_2samp, anderson_ksamp, norm, chi2_contingency, kstest
import os, sys, logging, time
from functools import partial

import cgs
import multi
import EPOS.analytics
from EPOS.fitfunctions import brokenpowerlaw1D
from EPOS.population import periodradius

try:
	import emcee
except ImportError:
	print '#WARNING# emcee could not be imported'

def once(epos, fac=1.0, Extra=None, goftype='KS'):
	'''
	Run EPOS once
	
	Description:
		Sets up some defaults and runs the MC simulation once (steps 1-4 in paper)
		
	Args:
		Extra: store the planet population as an extra for plotting, default None
	'''
	epos.goftype=goftype
	
	if not epos.Prep:
		
		print '\nPreparing EPOS run...'
		# prep the input population (this should be after MC run?)
		if epos.Parametric:
			fpar2d= epos.pdfpars.get2d(Init=True)
			print '  {} fit parameters'.format(len(fpar2d))
		
			# Call function once to see if it works, P=1, R=1
			try: 	epos.func(np.asarray([1.]), np.asarray([1.]), *fpar2d)
			except:	raise
			
			''' Check if all parameters are set, set defaults '''
			
			if epos.Multi:
				epos.pdfpars.default('f_cor',0.5)
				epos.pdfpars.default('f_iso',0.5)
				epos.pdfpars.default('inc',2)
				#epos.pdfpars.default('',)
				if epos.RandomPairing:
					epos.pdfpars.default('npl',5)
				else:
					epos.pdfpars.default('dR',0.1)
					if epos.spacing == 'brokenpowerlaw':
						epos.pdfpars.default('dP break',1.7)
						epos.pdfpars.default('dP 1',10)
						epos.pdfpars.default('dP 2',-3)
					elif epos.spacing == 'dimensionless':
						epos.pdfpars.default('log D',-0.3)
						epos.pdfpars.default('sigma',0.2)
					else:
						raise ValueError('no spacing defined')

		else:
			# set defaults for planet formation models here
			epos.fitpars.default('eta',0.5)
			epos.fitpars.default('f_cor',0.5)
			epos.fitpars.default('f_iso',0.5)
			epos.fitpars.default('f_inc',1.0)
			epos.fitpars.default('f_dP',1.0)

			if not epos.MassRadius and not 'R' in epos.pfm:
				raise ValueError('Supply radii or a mass-radius relation')
		
		# prep the detection efficiency / observations
		if not epos.Range: epos.set_ranges()
		prep_obs(epos) # make pdf, cdf
		epos.Prep=True
	
	''' set weights / parameters '''
	fpara= epos.fitpars.getfit(Init=True)
	
	''' Time the first MC run'''
	runtype= 'MC' if epos.MonteCarlo else 'noMC'
	if Extra is None:
		print '\nStarting the first {} run'.format(runtype)
	else:
		print '\nStarting extra {} run {}'.format(runtype, Extra)
	tstart=time.time()
	runonce= MC if epos.MonteCarlo else noMC
	runonce(epos, fpara, Store=True, Extra=Extra)
	tMC= time.time()
	print 'Finished one {} in {:.3f} sec'.format(runtype, tMC-tstart)
	epos.tMC= tMC-tstart
	
def mcmc(epos, nMC=500, nwalkers=100, dx=0.1, nburn=50, threads=1, npos=30, Saved=True):
	if not 'emcee' in sys.modules:
		raise ImportError('You need to install emcee')
	assert epos.Prep
	
	runonce= MC if epos.MonteCarlo else noMC
	
	''' set starting parameters '''
	fpara= epos.fitpars.getfit(Init=True)
		
	if not len(fpara)>0: raise ValueError('no fit paramaters defined')
	
	''' Load previous chain?'''
	ndim= len(fpara)
	shape= (nwalkers, nMC, ndim)
	
	# store, npy is uncompressed, savez returns as dict instead of array
	dir= 'chain/{}'.format(epos.name)
	fname= '{}/{}x{}x{}.npz'.format(dir, nwalkers, nMC, ndim)
	if not os.path.exists(dir): os.makedirs(dir)
	
	if os.path.isfile(fname) and Saved:
		print '\nLoading saved status from {}'.format(fname)
		npz= np.load(fname)
		
		epos.chain=npz['chain']
		assert epos.chain.shape == (nwalkers, nMC, ndim)
		
		if epos.seed!=npz['seed']: 
			print '\nNOTE: Random seed changed: {} to {}'.format(npz['seed'],epos.seed)
		epos.seed= npz['seed']
		
		# check if same keys (not very flexible)
		if 'fitkeys' in npz:
			for loadkey,key in zip(npz['fitkeys'],epos.fitpars.keysfit):
				if loadkey != key:
					raise ValueError('Stored key {} doesnt match {}'.format(loadkey,key))
	else:
		
		''' start the timer '''
		tstart=time.time()
		nsims= nMC*nwalkers
		runtime= (epos.tMC/3600.)*nsims # single-threaded run time
		print '\nPredicted runtime:'
		if runtime>1:
			print '  {:.3f} hours for {} runs at {:.3f} sec'.format(
					runtime, nsims, epos.tMC)
		else:
			print '  {:.1f} minutes for {} runs at {:.3f} sec'.format(
					runtime*60., nsims, epos.tMC)
		if threads > 1:
			if runtime/threads>1:
				print '  {:.3f} hours at 100% scaling'.format(runtime/threads)
			else:
				print '  {:.1f} minutes at 100% scaling'.format(runtime/threads*60.)
	
		''' Set up a log file '''
		# Note: if logging.debug is called before this, the log file is never created
		fdir= 'log/{}'.format(epos.name)
		if not os.path.exists(fdir): os.makedirs(fdir)
		flog='{}/{}.log'.format(fdir,time.strftime('%Y-%m-%d_%H.%M.%S'))
		logging.basicConfig(filename=flog,level=logging.DEBUG,filemode='w')
		logging.info('MCMC with random seed {}'.format(epos.seed))
		
		''' Wrap function '''
		lnmc= partial(runonce, epos, Verbose=False)
	
		''' Set up the MCMC walkers '''
		#p0 = [np.array(fpara)*np.random.uniform(1.-dx,1+dx,len(fpara)) 
		#		for i in range(nwalkers)]
		dx=np.array(epos.fitpars.getfit(attr='dx'))
		p0 = [np.array(fpara)+dx*np.random.uniform(-1,1,len(fpara)) 
				for i in range(nwalkers)]
		sampler = emcee.EnsembleSampler(nwalkers, len(fpara), lnmc, threads=threads)
	
		''' run the chain '''
		if True:
			# chop to pieces for progress bar?
			for i, result in enumerate(sampler.sample(p0, iterations=nMC)):
				amtDone= float(i)/nMC
				print '\r  [{:50s}] {:5.1f}%'.format('#' * int(amtDone * 50), amtDone * 100),
				os.sys.stdout.flush() 
		else:
			sampler.run_mcmc(p0, nMC)
		
		print '\nDone running\n'
		logging.info('Made it to the end')
		print 'Mean acceptance fraction: {0:.3f}'.format(
					np.mean(sampler.acceptance_fraction))

		''' Print run time'''	
		tMC= time.time()
		runtime= tMC-tstart
		if runtime>3600:
			print '  Runtime was {:.3f} hours at {:.3f} sec'.format(
					runtime/3600, (tMC-tstart)/nsims )
		else:
			print '  Runtime was {:.1f} minutes at {:.3f} sec'.format(
					runtime/60., (tMC-tstart)/nsims)
	
		epos.chain= sampler.chain
		print 'Saving status in {}'.format(fname)
		#np.save(fname, epos.chain)
		# compression slow on loading?
		np.savez_compressed(fname, chain=epos.chain, seed=epos.seed, keys=epos.fitpars.keysfit)
		
	''' the posterior samples after burn-in '''
	epos.samples= epos.chain[:, nburn:, :].reshape((-1, ndim))
	epos.burnin= nburn
	fitpars = map(lambda v: (v[1], v[2]-v[1], v[1]-v[0]),
                             zip(*np.percentile(epos.samples, [16, 50, 84],
                                                axis=0)))
	epos.fitpars.setfit([p[0] for p in fitpars])
	
	''' Generate posterior populations '''
	if npos is not None:
		epos.plotsample= epos.samples[np.random.randint(len(epos.samples), size=npos)]
		# run & store
		print '\nMC-ing the {} samples to plot'.format(npos)
		epos.ss_sample=[]
		for fpara in epos.plotsample:
			epos.ss_sample.append(\
				runonce(epos, fpara, Store=True, Sample=True, Verbose=False))
		# parallel
		# return & store in structure
	
	''' Estimate Solar System Analogs'''
	if epos.Parametric and epos.Multi:
		fMercury=[]
		fVenus=[]
		for sample in epos.samples:
			_, _, xpdf, _= periodradius(epos, fpara=sample)
			fMercury.append(np.sum(xpdf[epos.MC_xvar>88.])/np.sum(xpdf))
			fVenus.append(np.sum(xpdf[epos.MC_xvar>225.])/np.sum(xpdf))
		
		print
		for name, posterior in zip(['Mercury','Venus'],[fMercury, fVenus]):
			eta= np.percentile(posterior, [16, 50, 84]) 
			print '{} analogues < {:.1%} +{:.1%} -{:.1%}'.format(name, eta[1], 
					eta[2]-eta[1], eta[1]-eta[0])
			UL= np.percentile(posterior, [68.2, 95.4, 99.7])
			for i in range(3): print '  {} sigma UL {:.1%}'.format(i+1,UL[i])


	''' Best-fit values'''
	print '\nBest-fit values'
	for pname, fpar in zip(epos.fitpars.keysfit, fitpars): 
		print '  {}= {:.3g} +{:.3g} -{:.3g}'.format(pname,*fpar)

	print '\nStarting the best-fit MC run'	
	runonce(epos, np.array([p[0] for p in fitpars]), Store=True, BestFit=True)
	
def prep_obs(epos):
	# occurrence pdf on sma from plot_input_diag?

	# cdf of obs, zoomed only
	z=epos.obs_zoom={}
	ix= (epos.xzoom[0]<=epos.obs_xvar) & (epos.obs_xvar<=epos.xzoom[1])
	iy= (epos.yzoom[0]<=epos.obs_yvar) & (epos.obs_yvar<=epos.yzoom[1])
	
	if np.sum(ix&iy)==0:
		raise ValueError('No planets in the box!')
	
	x= epos.obs_xvar[ix&iy]
	y= epos.obs_yvar[ix&iy]

	for key, var in zip(['x','y'], [x,y]):
		z[key]= np.sort(var) # add (x0,0) and (x1,1)?
		z[key+' cum']= np.arange(z[key].size,dtype=float)
		z[key+' cdf']= z[key+' cum']/z[key+' cum'][-1]
	
	# multis
	z['multi']={}
	z['multi']['bin'], z['multi']['count']= multi.frequency(epos.obs_starID[ix&iy])
	z['multi']['pl cnt']= z['multi']['bin']*z['multi']['count']
	
	z['multi']['Pratio'], z['multi']['Pinner']= \
		multi.periodratio(epos.obs_starID[ix&iy], epos.obs_xvar[ix&iy])
	z['multi']['cdf']= multi.cdf(epos.obs_starID[ix&iy])

def MC(epos, fpara, Store=False, Sample=False, StorePopulation=False, Extra=None, 
		BestFit=False, Verbose=True):
	''' 
	Do the Monte Carlo Simulations
	Note:
	variable x/X is P
	variable y/Y is R/M
	TODO: split into multiple functions
	'''	
	if Verbose: tstart=time.time()
	#if not Store: logging.debug(' '.join(['{:.3g}'.format(fpar) for fpar in fpara]))
	
	''' Seed the random number generator '''
	if epos.seed is not None: np.random.seed(epos.seed)
	
	''' construct 1D arrays allP, allR or allM
	dimension equal to sample size * planets_per_star
	also keeping track of:
		ID: star identifier
		I: inc
		N: Nth planet in system
		dP: period ratio
	'''
	if epos.Parametric:
		
		''' parameters within bounds? '''
		try:
			epos.pdfpars.checkbounds(fpara)
		except ValueError as message:
			if Store: raise
			else:
				logging.debug(message)
				return -np.inf
				
		''' Draw (inner) planet from distribution '''
		pps= epos.fitpars.getpps_fromlist(fpara)
		fpar2d= epos.fitpars.get2d_fromlist(fpara)
		npl= epos.fitpars.getmc('npl', fpara) if epos.RandomPairing else 1
		
		try: sysX, sysY= draw_from_2D_distribution(epos, pps, fpar2d, npl=npl)
		except ValueError:
			if Store: raise
			else: return -np.inf
		
		''' Multi-planet systems '''
		if not epos.Multi:
			allX= sysX
			allY= sysY
			
		elif epos.RandomPairing:			
			# set ID, nth planet in system
			isys= np.arange(sysX.size/npl)
			allID= np.repeat(isys,npl)
			order= np.lexsort((sysX,allID)) # sort by ID, then P
			assert np.all(allID[order]==allID)

			allX= sysX[order]
			allY= sysY #[order] # random order anyway
			allN= np.tile(np.arange(npl),isys.size) # ignores xzoom
			
			# isotropic or inc?
			dInc= epos.fitpars.getmc('inc', fpara)
			if dInc is not None:
				f_iso= epos.fitpars.getmc('f_iso', fpara)
				allI= np.random.rayleigh(dInc, allID.size)
			
			f_cor= epos.fitpars.getmc('f_cor', fpara)
			f_dP, f_inc= 1.0, 1.0 # no need to fudge these
			
		else:
			''' retrieve fit parameters for multi-planets'''
			npl= epos.fitpars.getmc('npl', fpara)
			dR= epos.fitpars.getmc('dR', fpara)
			dInc= epos.fitpars.getmc('inc', fpara)
			f_iso= epos.fitpars.getmc('f_iso', fpara)
			f_cor= epos.fitpars.getmc('f_cor', fpara)
			
			f_dP, f_inc= 1.0, 1.0 # no need to fudge these

			
			''' Parameter bounds '''
			if npl < 1:
				logging.debug('npl = {:.3g} < 1'.format(npl))
				if Store: raise ValueError('at least one planet per system')
				return -np.inf
			if (dInc <=0) or (dR <=0) or not (0 <= f_iso <= 1):
				if Store: raise ValueError('parameters out of bounds')
				return -np.inf
			
			''' Draw multiplanet distributions '''
			allX, allY, allI, allN, allID= \
				draw_multi(epos, sysX, sysY, npl, dInc, dR, fpara, Store)
					
		''' convert to observable parameters '''
		allP= allX
		if epos.RV or epos.MassRadius: allM= allY
		else:		allR= allY
				
	else:
		pfm= epos.pfm
			
		''' parameters within bounds? '''
		# move out of loop?
		try:
			epos.fitpars.checkbounds(fpara)
		except ValueError as message:
			if Store: raise
			else:
				logging.debug(message)
				return -np.inf
				
		''' Fit parameters'''
		pps= epos.fitpars.getpps_fromlist(fpara)

		f_cor= epos.fitpars.getmc('f_cor', fpara)
		f_iso= epos.fitpars.getmc('f_iso', fpara)
		f_inc= epos.fitpars.getmc('f_inc', fpara)
		f_dP= epos.fitpars.getmc('f_dP', fpara)
					
		# need this here?
		if not (0<=f_iso<=1) or not (0 < pps) or not (0 <= f_cor <= 1):
			#\or not (0<=f_dP<=10) or not (0 <= f_inc < 10):
			if Store: raise ValueError('parameters out of bounds')
			return -np.inf
		
		''' 
		Draw from all
		'''
		if not 'draw prob' in pfm:
			ndraw= int(round(1.*epos.nstars*pps/pfm['ns']))
			if Verbose: 
				print '  {} planets in {} simulations'.format(pfm['np'],pfm['ns'])
				print '  {} stars in survey, {} draws, eta={:.2g}'.format(epos.nstars, ndraw, pps)
		
			allP= np.tile(pfm['P'], ndraw)
			allM= np.tile(pfm['M'], ndraw)
			if 'R' in pfm:
				allR= np.tile(pfm['R'], ndraw)
		
			if epos.Multi:
				allI= np.tile(pfm['inc'], ndraw)
				allN= np.tile(pfm['kth'], ndraw)
				
				# ID or system index?
				allID= np.tile(pfm['ID'], ndraw) \
						+ np.repeat(np.arange(ndraw)*pfm['ns'], pfm['np'])
		else:
			'''
			Draw from some distributions according to 'tag' parameter
			TODO: functions to calculate draw probability from tag
			'''
			#draw planetary systems from simulations
			ndraw= int(round(1.*epos.nstars*pps))
			if Verbose: print '\nDraw {} systems'.format(ndraw) 
			system_index= np.random.choice(pfm['system index'], size=ndraw, 
							p=pfm['draw prob'])
			
			#create a list of planets
			indexlist= [pfm['planet index'][pfm['ID']==i] for i in system_index]
			starIDlist= [np.full((pfm['ID']==i).sum(),ID) 
							for ID,i in enumerate(system_index)]
			
			planets= np.concatenate(indexlist)
			allID= np.concatenate(starIDlist)
			allP= pfm['P'][planets]
			allM= pfm['M'][planets]
			allR= pfm['R'][planets]
			allI= pfm['inc'][planets]
			allN= pfm['kth'][planets]
			#allID= pfm['ID'][planets]
			
			if Verbose: print '  {} planets'.format(allP.size)
		
		allY= allM
			
		dInc=False # Isotropic inclinations not implemented
			
	''' 
	Identify transiting planets (itrans is a T/F array)
	'''
	if epos.RV:
		# RV keep all
		itrans= np.full(allP.size, True, np.bool)
	elif (not epos.Multi) or (epos.Multi and dInc==None):
		# geometric transit probability
		p_trans= epos.fgeo_prefac *allP**epos.Pindex
		itrans= p_trans >= np.random.uniform(0,1,allP.size)
	else:
		#multi-transit probability
		itrans= istransit(epos, allID, allI, allP, f_iso, f_inc, Verbose=Verbose)
		
	# Print multi statistics	
	if Verbose and epos.Multi and not epos.RV: 
		print '\n  {} planets, {} transit their star'.format(itrans.size, itrans.sum())
		multi.frequency(allID[itrans], Verbose=True)
	
	'''
	remove planets according to transit probability
	'''	
	MC_P= allP[itrans]
	if epos.MassRadius or epos.RV:	MC_M= allM[itrans]
	else:							MC_R= allR[itrans]	
	if epos.Multi:
		MC_ID= allID[itrans]	
		if epos.Parametric:
			MC_N= allN[itrans] # also for PFM?
		else:
			#MC_P*= (1.+0.1*np.random.normal(size=MC_ID.size) )
			pass

	'''
	Set the observable MC_Y (R or Msin i) 
	'''
	if epos.RV:
		''' M sin i.'''
		# Note different conventions for i in Msini (i=0 is pole-on)
		# sin(arccos(chi)) == cos(arcsin(chi)) == sqrt(1-chi^2)
		MC_Y= MC_Msini= MC_M* np.sqrt(1.-np.random.uniform(0,1,MC_P.size)**2.)
	else:
		''' Convert Mass to Radius '''
		if epos.MassRadius:
			mean, dispersion= epos.MR(MC_M)
			MC_R= mean+ dispersion*np.random.normal(size=MC_M.size)
		
		''' uncertainty in stellar radius? '''
		MC_Y=MC_R * (1.+epos.radiusError*np.random.normal(size=MC_R.size) )

	
	'''
	Store (transiting) planet sample for verification plot
	'''
	if Store:
		tr= epos.transit={}
		tr['P']= MC_P
		tr['Y']= MC_Y
	
	'''
	Identify detectable planets based on SNR (idet is a T/F array)
	'''
	f_snr= interpolate.RectBivariateSpline(epos.MC_xvar, epos.MC_yvar, epos.MC_eff)
	p_snr= f_snr(MC_P, MC_Y, grid=False)
	assert p_snr.ndim == 1

	idet= p_snr >= np.random.uniform(0,1,MC_P.size)
	
	# draw same random number for S/N calc, 1=correlated noise
	if epos.Multi and f_cor >0:
		IDsys, toplanet= np.unique(MC_ID, return_inverse=True) 
		idet_cor= p_snr >= np.random.uniform(0,1,IDsys.size)[toplanet]

		cor_sys= (np.random.uniform(0,1,IDsys.size) < f_cor)
		cor_pl= cor_sys[toplanet]
		idet = np.where(cor_pl, idet_cor, idet)

	''' 
	Remove undetectable planets
	'''
	# arrays with detected planets
	if epos.Multi:
		det_ID= MC_ID[idet]
		if not epos.RV and epos.Parametric: det_N= MC_N[idet]
	det_P= MC_P[idet]
	det_Y= MC_Y[idet]

	#if len(alldP)>0: 
	# 	if not epos.Parametric and 'all_Pratio' in sg: 
	# 		det_dP= MC_dP[idet]

	if Verbose and epos.Multi: 		
		print '  {} transiting planets, {} detectable'.format(idet.size, idet.sum())
		multi.frequency(det_ID, Verbose=True)

	'''
	Probability that simulated data matches observables
	TODO:
	- store ln prob, D
	- switches for in/excluding parameters
	- likelyhood from D instead of prob?
	'''
	# bugfix 
	#np.seterr(divide='raise')

	tstart=time.time()
	prob={}
	lnp={}
	
	# make sure that x=P, y=R (where?)
	ix= (epos.xzoom[0]<=det_P) & (det_P<=epos.xzoom[1])
	iy= (epos.yzoom[0]<=det_Y) & (det_Y<=epos.yzoom[1])
	if (ix&iy).sum()<1:
		if Store: raise ValueError('no planets detectable')
		return -np.inf			
	
	if epos.goftype=='KS':
		prob_2samp= _prob_ks
	elif epos.goftype=='AD': 
		prob_2samp= _prob_ad
	else:
		raise ValueError('{} not a goodness-of-fit type (KS, AD)'.format(epos.goftype))

	if 'xvar' in epos.summarystatistic:
		prob['xvar'], lnp['xvar']=  prob_2samp(epos.obs_zoom['x'], det_P[ix&iy])
	if 'yvar' in epos.summarystatistic:
		prob['yvar'], lnp['yvar']=  prob_2samp(epos.obs_zoom['y'], det_Y[ix&iy])

	if 'N' in epos.summarystatistic:
		# chi^2: (np-nobs)/nobs**0.5 -> p: e^-0.5 x^2
		chi2= (epos.obs_zoom['x'].size-np.sum(ix&iy))**2. / epos.obs_zoom['x'].size
		lnp['N']= -0.5* chi2
		prob['N']= np.exp(-0.5* chi2)
	
	if epos.Multi:
		''' Multi-planet frequency, pearson chi_squared '''
		k, Nk= multi.frequency(det_ID[ix&iy])
		Nk_obs= epos.obs_zoom['multi']['count']
		ncont= max(len(Nk),len(Nk_obs))
		
		# pad with zeros
		obs= np.zeros((2,ncont), dtype=int)
		obs[0,:len(Nk)]= Nk
		obs[1,:len(Nk_obs)]= Nk_obs
		# remove double zero frequencies
		obs=obs[:, ~((obs[0,:] == 0) & (obs[1,:]==0))]
		
		try:	
			_, prob['Nk'], _, _ = chi2_contingency(obs)
		except ValueError:
			#print Nk
			#print Nk_obs
			#print obs
			raise
			
		with np.errstate(divide='ignore'): lnp['Nk']= np.log(prob['Nk'])			
		
		''' Period ratio, innermost planet '''
		sim_dP, sim_Pinner= multi.periodratio(det_ID[ix&iy], det_P[ix&iy])

		if (len(sim_dP)>0) & (len(sim_Pinner)>0): 
			prob['dP'],lnp['dP']= prob_2samp(epos.obs_zoom['multi']['Pratio'],f_dP*sim_dP)					
			prob['Pin'],lnp['Pin']= prob_2samp(epos.obs_zoom['multi']['Pinner'],
											sim_Pinner)
		else:
			logging.debug('no multi-planet statistics, {}'.format(len(sim_dP)))
			prob['dP'], prob['Pin']= 0, 0 
			lnp['dP'], lnp['Pin']= -np.inf, -np.inf

	''' combine with Fischer's rule: '''
	lnprob= np.sum([lnp[key] for key in epos.summarystatistic])
	#dof= len(prob_keys)
	chi_fischer= -2. * lnprob

	if Verbose:
		print '\nGoodness-of-fit'
		print '  logp= {:.1f}'.format(lnprob)
		print '  - p(n={})={:.2g}'.format(np.sum(ix&iy), prob['N'])
		if 'xvar' in prob:	print '  - p(x)={:.2g}'.format(prob['xvar'])
		if 'yvar' in prob:	print '  - p(y)={:.2g}'.format(prob['yvar'])
		if 'Nk' in prob:	print '  - p(N_k)={:.2g}'.format(prob['Nk'])
		if 'dP' in prob:	print '  - p(P ratio)={:.2g}'.format(prob['dP'])
		if 'Pin' in prob:	print '  - p(P inner)={:.2g}'.format(prob['Pin'])

		if BestFit: 
			''' Akaike/Bayesian information criterion'''
			k_free= epos.fitpars.get_kfree()
			n_data= epos.obs_zoom['x'].size
			bic= EPOS.analytics.bic_loglike(lnprob, k_free, n_data)
			aic= EPOS.analytics.aic_loglike(lnprob, k_free)
			aic_c= EPOS.analytics.aic_c_loglike(lnprob, k_free, n_data)

			print '\n  Akaike/Bayesian Information Criterion'
			print '  - k={}, n={}'.format(k_free,n_data)
			print '  - BIC= {:.1f}'.format(bic)
			print '  - AIC= {:.1f}, AICc= {:.1f}'.format(aic,aic_c)
		
	tgof= time.time()
	if Verbose: print '  observation comparison in {:.3f} sec'.format(tgof-tstart)

	''' Store _systems_ with at least one detected planet '''
	# StorePopulation
	if Store and epos.Multi and (not epos.RV):
		# include/exclude singles w/ iso_pl?
		itransdet= np.copy(itrans)
		itransdet[itrans]= idet
		isysdet,isingle,imulti,order= storepopulation(allID, allP, det_ID, itransdet)
				
		pop= epos.population={}
		pop['order']= order
		pop['P']= allP
		pop['k']= allN
		pop['inc']= allI
		for key, subset in zip(['system', 'single', 'multi'],[isysdet, isingle, imulti]):
			pop[key]={}
			pop[key]['Y']= allY[subset] # R? M?
			pop[key]['P']= allP[subset]
			pop[key]['ID']= allID[subset]
			pop[key]['inc']= allI[subset]
			pop[key]['order']= order[subset]
			pop[key]['detectable']= itransdet[subset]
	
	''' Store detectable planet population '''	
	if Store:
		ss={}
		ss['P']= det_P # MC_P[idet]
		ss['Y']= det_Y
		if epos.MassRadius:
			# or if has mass and radius
			ss['M']= MC_M[idet]
			ss['R']= MC_R[idet]
		
		#if len(alldP)>0
		if epos.Multi:
			ss['ID']= det_ID
			ss['multi']={}
			ss['multi']['bin'], ss['multi']['count']= multi.frequency(det_ID[ix&iy])
			ss['multi']['pl cnt']=ss['multi']['bin']* ss['multi']['count']
			ss['multi']['Pratio'], ss['multi']['Pinner']= \
				multi.periodratio(det_ID[ix&iy], det_P[ix&iy]) # *f_dP
			ss['multi']['cdf']= multi.cdf(det_ID[ix&iy])
			if not epos.RV and epos.Parametric:
				ss['multi']['PN'],ss['multi']['dPN']= multi.periodratio(
						det_ID[ix&iy], det_P[ix&iy], N=det_N[ix&iy]) 
		
		epos.prob=prob
		epos.lnprob=lnprob
		ss['P zoom']= det_P[ix&iy]
		ss['Y zoom']= det_Y[ix&iy]
		ss['nobs']= ss['P zoom'].size
		
		# Store as an extra model 
		if Sample:
			return ss
		elif Extra is not None:
			ss['name']=Extra
			if not hasattr(epos,'ss_extra'):
				epos.ss_extra=[]
			epos.ss_extra.append(ss)
			#print 'saving extra {}'.format(Extra)
		else:
			epos.synthetic_survey= ss 
	else:
		# return probability
		if np.isnan(lnprob):
			return -np.inf
		return lnprob

def noMC(epos, fpara, Store=False, Sample=False, StorePopulation=False, Extra=None, 
		BestFit=False, Verbose=True):
	''' 
	Do the Simulations without Monte Carlo
	'''	
	if Verbose: tstart=time.time()
	#if not Store: logging.debug(' '.join(['{:.3g}'.format(fpar) for fpar in fpara]))

	if epos.Multi: raise ValueError('Multi-planets need Monte Carlo (?)')
	if not epos.Parametric: 
		raise ValueError('Planet Formation models need Monte Carlo (?)')
		
	''' parameters within bounds? '''
	try:
		epos.pdfpars.checkbounds(fpara)
	except ValueError as message:
		if Store: raise
		else:
			logging.debug(message)
			return -np.inf

	''' Generate observable period-radius distribution, in counts'''
	if epos.RV:
		pps, pdf, pdf_X, pdf_Y= periodradius(epos, fpara=fpara, fdet=epos.MC_eff*epos.nstars, 
			Convert=epos.Msini)
	elif epos.MassRadius:
		raise ValueError('Generate pdf on radius grid here')
	else:
		pps, pdf, pdf_X, pdf_Y= periodradius(epos, fpara=fpara, fdet=epos.f_det*epos.nstars)

	'''
	Probability that simulated data matches observables
	'''
	tstart=time.time()
	prob={}
	lnp={}
	
	pdf_x= np.interp(epos.noMC_zoom_x, epos.MC_xvar, pdf_X)
	pdf_y= np.interp(epos.noMC_zoom_y, epos.MC_yvar, pdf_Y)
	cdf_x= np.cumsum(pdf_x)
	cdf_y= np.cumsum(pdf_y)
	cdf_x/= cdf_x[-1]
	cdf_y/= cdf_y[-1] 
	
	''' Wrap functions '''
	func_cdf_X= partial(np.interp, xp=epos.noMC_zoom_x, fp=cdf_x, left=0, right=0)
	func_cdf_Y= partial(np.interp, xp=epos.noMC_zoom_y, fp=cdf_y, left=0, right=0)
		 	
	prob['xvar'], lnp['xvar']= _prob_ks_func(epos.obs_zoom['x'], func_cdf_X)
	prob['yvar'], lnp['yvar']= _prob_ks_func(epos.obs_zoom['y'], func_cdf_Y)

	#nobs= int(pdf[epos.trim_to_zoom].sum())
	nobs_x=int(np.sum(pdf_x)/epos.noMC_scale_x)
	nobs_y=int(np.sum(pdf_y)/epos.noMC_scale_y)
	nobs= int(np.sqrt(nobs_x*nobs_y))
	if Verbose: print 'nobs={} (x:{},y:{})'.format(nobs,nobs_x,nobs_y) # ??

	chi2= (epos.obs_zoom['x'].size-nobs)**2. / epos.obs_zoom['x'].size
	lnp['N']= -0.5* chi2
	prob['N']= np.exp(-0.5* chi2)
		
	prob_keys= ['N','xvar','yvar']

	# combine with Fischer's rule:
	lnprob= np.sum([lnp[key] for key in prob_keys])
	#dof= len(prob_keys)
	chi_fischer= -2. * lnprob
	
	
	if Verbose:
		print '\nGoodness-of-fit'
		print '  logp= {:.1f}'.format(lnprob)
		print '  - p(n={})={:.2g}'.format(nobs, prob['N'])
		print '  - p(x)={:.2g}'.format(prob['xvar'])
		print '  - p(y)={:.2g}'.format(prob['yvar'])

		if BestFit:
			''' Bayesian information criterion'''
			k_free= epos.fitpars.get_kfree()
			n_data= epos.obs_zoom['x'].size
			bic= EPOS.analytics.bic_loglike(lnprob, k_free, n_data)
			aic= EPOS.analytics.aic_loglike(lnprob, k_free)
			aic_c= EPOS.analytics.aic_c_loglike(lnprob, k_free, n_data)

			print '\n  Akaike/Bayesian Information Criterion'
			print '  - k={}, n={}'.format(k_free,n_data)
			print '  - BIC= {:.1f}'.format(bic)
			print '  - AIC= {:.1f}, AICc= {:.1f}'.format(aic,aic_c)
		
	tgof= time.time()
	if Verbose: print '\nObservation comparison in {:.3f} sec'.format(tgof-tstart)

	''' Store detectable planet population '''	
	if Store:
		ss={}
		ss['nobs']= nobs
		
		ss['pdf']= pdf

		ss['P']= epos.MC_xvar
		ss['P pdf']= pdf_X
		
		ss['Y']= epos.MC_yvar
		ss['Y pdf']= pdf_Y
		
		ss['P zoom']= epos.noMC_zoom_x
		ss['P zoom pdf']= pdf_x # /scale?
		ss['P zoom cdf']= cdf_x

		ss['Y zoom']= epos.noMC_zoom_y
		ss['Y zoom pdf']= pdf_y # /scale?
		ss['Y zoom cdf']= cdf_y
		
		epos.prob=prob
		epos.lnprob=lnprob

		if Sample:
			return ss
		elif Extra is not None:
			ss['name']=Extra
			if not hasattr(epos,'ss_extra'):
				epos.ss_extra=[]
			epos.ss_extra.append(ss)
			#print 'saving extra {}'.format(Extra)
		else:
			epos.synthetic_survey= ss
	else:
		''' return probability '''
		if np.isnan(lnprob):
			return -np.inf
		return lnprob
	
def draw_from_2D_distribution(epos, pps, fpara, npl=1):
	
	''' create PDF, CDF'''
	# assumes a separable function of mass and radius
	pdf= epos.func(epos.X_in, epos.Y_in, *fpara)

	pdf_X, pdf_Y= np.sum(pdf, axis=1), np.sum(pdf, axis=0)
	cum_X, cum_Y= np.cumsum(pdf_X), np.cumsum(pdf_Y)
	#pps_x, pps_y=  cum_X[-1], cum_Y[-1]
	#planets_per_star= 0.5*(pps_x+pps_y) # should be equal
	
	try:
		ndraw= npl*int(round(pps*epos.nstars))
	except OverflowError:
		logging.debug('infinity encountered, pps= {},{}'.format(pps_x, pps_y))
		for pname, fpar in zip(epos.pname, fpara):
			logging.debug('  {}= {:.3g}'.format(pname,fpar))
		raise ValueError('Infinity encountered')
	
	if ndraw < 1: 
		logging.debug('no draws ({}, {}*{})'.format(ndraw, epos.nstars, planets_per_star))
		raise ValueError('no planets')
	elif ndraw > 1e8:
		logging.debug('>1e8 planets ({})'.format(ndraw))
		raise ValueError('too many planets')
	# 	elif planets_per_star > 100:
	# 		logging.debug('>100 planets per star ({})'.format(planets_per_star))
	# 		raise ValueError('too many planets per star')
	try:
		allX= np.interp(np.random.uniform(cum_X[0],cum_X[-1],ndraw), cum_X, epos.MC_xvar)
		allY= np.interp(np.random.uniform(cum_Y[0],cum_Y[-1],ndraw), cum_Y, epos.in_yvar)
	except MemoryError:
		raise ValueError('Memory error for n={}'.format(ndraw))
	except OverflowError:
		print ndraw
		print cum_X
		print cum_Y
		raise
	
	return allX, allY

def draw_multi(epos, sysX, sysY, npl, dInc, dR, fpara, Store):
	''' assign ID to each system '''
	sysID= np.arange(sysX.size)
	#allID= np.repeat(sysID, npl) # array with star ID for each planet
	npl_arr= np.random.uniform(npl, npl+1, sysID.size) # rounds down
	allID= np.repeat(sysID, npl_arr.astype(int))
	#print allID.size, sysID.size
	if allID.size > 1e7:
		logging.debug('Too many planets: {} > 1e7'.format(allID.size))
		for pname, fpar in zip(epos.pname, fpara):
			logging.debug('  {}= {:.3g}'.format(pname,fpar))
		if Store: raise ValueError('Too many planets: {}'.format(allID.size))
		return -np.inf				

	''' initialize planet parameters'''
	_, toplanet, sysnpl= np.unique(allID, return_inverse=True,return_counts=True)
	allX= sysX[toplanet]
	allY= sysY[toplanet]
	allI= np.random.rayleigh(dInc, allID.size)
	#allN= np.ones_like(allID) # index to planet in system
	allN= np.where(allX>=epos.xzoom[0],1,0) # also yzoom?
	#print allX[:3]

	# get index of first planet in each system
	if len(sysnpl) < 1:
		logger.debug('no planets')
		if Store: raise ValueError('no planets')
		return -np.inf
	di= np.roll(sysnpl,1)
	di[0]=0
	i1= np.cumsum(di) # index to first planet
	assert np.all(sysID == allID[i1])
	
	''' Draw period of 2nd, 3rd planet etc.'''
	Pgrid= np.logspace(0,1)
	# use population.periodratio here
	if epos.spacing == 'powerlaw':
		dPbreak= epos.fitpars.getmc('dP break', fpara)
		dP1= epos.fitpars.getmc('dP 1', fpara)
		dP2= epos.fitpars.getmc('dP 2', fpara)
		if (dPbreak<=0):
			if Store: raise ValueError('parameters out of bounds')
			return -np.inf
		cdf= np.cumsum(brokenpowerlaw1D(Pgrid, dPbreak, dP1, dP2))
	elif epos.spacing=='dimensionless':
		logD=  epos.fitpars.getmc('log D', fpara)
		sigma= epos.fitpars.getmc('sigma', fpara)
		if (sigma<=0):
			if Store: raise ValueError('parameters out of bounds')
			return -np.inf			

		with np.errstate(divide='ignore'): 
			Dgrid= np.log10(2.*(Pgrid**(2./3.)-1.)/(Pgrid**(2./3.)+1.))
		Dgrid[0]= -2
		#print Dgrid
		cdf= norm(logD,sigma).cdf(Dgrid)
	
	# loop over planet 2,3... n
	for i in range(2,len(np.bincount(sysnpl)) ):
		im= i1[sysnpl>=i] # index to ith planet in each system
		#print '  planet {} in system, {} systems'.format(i,im.size)
		#dP= draw_from_function(brokenpowerlaw1D,xgrid,im.size, dPbreak, dP1, dP2)
		dP= np.interp(np.random.uniform(cdf[0],cdf[-1],im.size), cdf, Pgrid)
		allX[im+(i-1)]= allX[im+(i-2)]* dP
		#np.random.uniform(dP-0.7, dP+0.7, im.size)
		#print allX[:3]
		#np.random.norm()
		allY[im+(i-1)]= allY[im+(i-2)]* 10.**np.random.normal(0, dR, im.size)
		
		# nth planet in system (zoom range)
		#allN[im+(i-1)]= allN[im+(i-2)]+1
		#allN[im+(i-1)]= allN[im+(i-2)]+np.where(allX[im+(i-1)]>=epos.xzoom[0],1,0)
		allN[im+(i-1)]= np.where(allX[im+(i-1)]>=epos.xzoom[0],allN[im+(i-2)]+1,0)

		
	#print '+{}={} story checks out?'.format(allID[i1].size, allID.size)
	#print allX[:3]
	
	''' Toss out planets (reduces memory footprint) '''
	Xinrange= allX<=epos.MC_xvar[-1]
	#if Xinrange.sum() < 0.5* allX.size: 
	#	print 'yeah, {}/{}'.format(Xinrange.sum(),allX.size)
	allX= allX[Xinrange]
	allY= allY[Xinrange]
	allI= allI[Xinrange]
	allN= allN[Xinrange]
	allID= allID[Xinrange]
	
	return allX, allY, allI, allN, allID
	
def istransit(epos, allID, allI, allP, f_iso, f_inc, Verbose=False):
	# draw same numbers for multi-planet systems
	IDsys, toplanet= np.unique(allID, return_inverse=True) # return_counts=True
	if Verbose: print '  {}/{} systems'.format(IDsys.size, allID.size)
	
	# draw system viewing angle proportionate to sin theta (i=0: edge-on)
	inc_sys= np.arcsin(np.random.uniform(0,1,IDsys.size))
	inc_pl= inc_sys[toplanet]
	assert inc_pl.size == allP.size
	
	R_a= epos.fgeo_prefac *allP**epos.Pindex # == p_trans
	mutual_inc= allI * f_inc
	#mutual_inc= 0.0 # planar distribution
	#mutual_inc= 1.0 # fit 
	if Verbose: 
		print '  Average mutual inc={:.1f} degrees'.format(np.median(allI))
		if f_inc != 1.0:
			print 'f_inc= {:.2g}, inc= {:.1f} deg'.format(f_inc, np.median(mutual_inc))
	delta_inc= mutual_inc *np.cos(np.random.uniform(0,np.pi,allP.size)) * np.pi/180.
	itrans= np.abs(inc_pl+delta_inc) < np.arcsin(R_a)

	# allow for a fraction of isotropic systems
	if f_iso > 0:
		p_trans= epos.fgeo_prefac *allP**epos.Pindex
		itrans_iso= p_trans >= np.random.uniform(0,1,allP.size)
		
		iso_sys= (np.random.uniform(0,1,IDsys.size) < f_iso)
		iso_pl= iso_sys[toplanet]
		itrans = np.where(iso_pl, itrans_iso, itrans)
		
	return itrans

def storepopulation(allID, allP, det_ID, idetected):
	# add f_iso?

	# systems with at least one detectable planet
	try:
		isysdet= np.isin(allID, det_ID)
	except AttributeError:
		isysdet= np.in1d(allID, det_ID)
	
	# detected multis/singles, index to allID
	uniqueID, detcounts= np.unique(allID[idetected],return_counts=True)
	IDmulti= uniqueID[(detcounts>1)] # systems that are multi
	IDsingle= uniqueID[(detcounts==1)]
	# returns all planets in system (not just detectable ones)
	try:
		imulti=  np.isin(allID, IDmulti)
		isingle= np.isin(allID, IDsingle)
	except AttributeError:
		imulti=  np.in1d(allID, IDmulti)
		isingle= np.in1d(allID, IDsingle)

	# Sort order [0-1] based on inner period (also of undetected)
	IDsys, iunique, toplanet= np.unique(allID, return_index=True, return_inverse=True)
	assert np.all(IDsys[toplanet] == allID)
	Pinner= allP[iunique] # inner if lexsorted
	
	idx= np.argsort(Pinner) #[::-1]
	rank= np.argsort(idx)
	
	#Porder= rank[toplanet]
	Porder= 1.0 * rank[toplanet] / IDsys.size
 
	return isysdet, isingle&idetected, imulti&idetected, Porder

def _prob_ks_func(a,func):
	_, prob= kstest(a,func)
	with np.errstate(divide='ignore'):
		lnprob= np.log(prob)
	return prob, lnprob

def _prob_ks(a,b):
	_, prob= ks_2samp(a,b)
	with np.errstate(divide='ignore'):
		lnprob= np.log(prob)
	return prob, lnprob

def _prob_ad(a,b):
	_, _, prob= anderson_ksamp([a,b])
	with np.errstate(divide='ignore'):
		lnprob= np.log(prob)
	if prob > 1:
		print
		print anderson_ksamp([a,b])
		print ks_2samp(a,b)
		print
		print a
		print b
		print prob, lnprob
	return prob, lnprob

''' Old code '''
# def draw_from_function(f, grid, ndraw, *args):
# 	cdf= np.cumsum(f(grid, *args))
# 	return np.interp(np.random.uniform(0,cdf[-1],ndraw), cdf, grid)

#def make_pdf(epos, norm=None, Init=False):


#! /usr/bin/env ipython
''' 
Example script fitting a power-law in period and M sin i to radial velocity data
'''
import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='example_5', RV=True, MC=False, Msini=True)

''' load the exoplanets and completeness from Mayor+ 2011, Fernandes+ 2018'''
obs, survey= EPOS.rv.Mayor2011()
epos.set_observation(**obs)
epos.set_survey(**survey)

''' Define a double broken power-law as a planet population '''
epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D)

''' Parameter initial guess and fitting ranges
Note: 
	- brokenpowerlaw2D uses 6 parameters, indicated with the is2D keyword
	- 'pps' is a normalization factor for the planet occurrence rate (planets-per-star)
	- parameters a_M and b_M are not fitted
	- dx is the range in walker initial positions for parameters that change sign (+/-)
'''
epos.fitpars.add('pps',		1.0, 	min=1e-3)
epos.fitpars.add('P break',	1e3,	min=100,max=7e3,is2D=True)
epos.fitpars.add('a_P',		1.0, 	min=0,	max=3, 	is2D=True)
epos.fitpars.add('b_P',		-0.5,	min=-3, max=0,	is2D=True)
epos.fitpars.add('M break',	10.0,	fixed=True, 	is2D=True) 
epos.fitpars.add('a_M',		0.0,	fixed=True, 	is2D=True)
epos.fitpars.add('b_M',		-0.5,	dx=0.1,		 	is2D=True)

''' define the simulated range (trim) and the range compared to observations (zoom)'''
epos.set_ranges(xtrim=[1,1e5],ytrim=[1,1e5],xzoom=[10,1e4],yzoom=[50,1e4], Occ=True)

''' Run the Monte Carlo Simulation once '''
EPOS.run.once(epos)

''' run an MCMC chain on multiple cores or read in a previously saved run'''
EPOS.run.mcmc(epos, nMC=1000, nwalkers=100, nburn=200, threads=20, Saved=True)

''' define bins where occurrence is calculated'''
#epos.set_bins(xbins=[[2,400],[0.9*365,2.2*365]], ybins=[[1,6],[0.7,1.5]]) # eta_zoom, eta_earth

''' Calculate the occurrence rates '''
EPOS.occurrence.all(epos)

''' Adjust plot parameters'''
epos.plotpars['occrange']= [2e-4,2.]

''' plot everything '''
EPOS.plot.survey.all(epos)
EPOS.plot.input.all(epos)
EPOS.plot.output.all(epos)
EPOS.plot.mcmc.all(epos)
EPOS.plot.occurrence.all(epos)